@@ -569,7 +569,7 @@ def go(sympy_top, file, verbose=False, no_color=False, exact=True, sphinx=True):
             total_sphinx += _total_sphinx
             num_functions += _num_functions
         return doctests, total_sphinx, num_functions
-    if (not (file.endswith('.py') or file.endswith('.pyx')) or
+    if (not (file.endswith((".py", ".pyx"))) or
         file.endswith('__init__.py') or
         not exact and ('test_' in file or 'bench_' in file or
         any(name in file for name in skip_paths))):
