@@ -3,15 +3,14 @@
 """
 Plotting Examples
 
-Suggested Usage:    python -i plotting.py
+Suggested Usage:    python -i pyglet_plotting.py
 """
 
 
-from sympy import symbols
+from sympy import symbols, sin, cos, pi, sqrt
 from sympy.plotting.pygletplot import PygletPlot
-from sympy import sin, cos, pi, sqrt, exp
 
-from time import sleep, clock
+from time import sleep, perf_counter
 
 
 def main():
@@ -127,17 +126,17 @@ def main():
 
     @example_wrapper
     def lambda_vs_sympy_evaluation():
-        start = clock()
+        start = perf_counter()
         p[4] = x**2 + y**2, [100], [100], 'style=solid'
         p.wait_for_calculations()
-        print("lambda-based calculation took %s seconds." % (clock() - start))
+        print("lambda-based calculation took %s seconds." % (perf_counter() - start))
 
-        start = clock()
+        start = perf_counter()
         p[4] = x**2 + y**2, [100], [100], 'style=solid; use_sympy_eval'
         p.wait_for_calculations()
         print(
             "sympy substitution-based calculation took %s seconds." %
-            (clock() - start))
+            (perf_counter() - start))
 
     @example_wrapper
     def gradient_vectors():
@@ -157,9 +156,9 @@ def main():
                 FG = lambdify([x, y], [dx, dy, dz])
                 iu.v_steps /= 5
                 iv.v_steps /= 5
-                Gvl = list(list([FF(u, v), FG(u, v)]
-                                for v in iv.frange())
-                           for u in iu.frange())
+                Gvl = [[[FF(u, v), FG(u, v)]
+                                for v in iv.frange()]
+                           for u in iu.frange()]
 
                 def draw_arrow(p1, p2):
                     """
@@ -195,9 +194,9 @@ def main():
 
     def help_str():
         s = ("\nPlot p has been created. Useful commands: \n"
-             "    help(p), p[1] = x**2, print p, p.clear() \n\n"
+             "    help(p), p[1] = x**2, print(p), p.clear() \n\n"
              "Available examples (see source in plotting.py):\n\n")
-        for i in xrange(len(examples)):
+        for i in range(len(examples)):
             s += "(%i) %s\n" % (i, examples[i].__name__)
         s += "\n"
         s += "e.g. >>> example(2)\n"
