@@ -1,6 +1,8 @@
-from typing import Dict, Callable
+from __future__ import annotations
+from typing import Callable
 
-from sympy.core import S, Add, Expr, Basic, Mul
+from sympy.core import S, Add, Expr, Basic, Mul, Pow, Rational
+from sympy.core.logic import fuzzy_not
 from sympy.logic.boolalg import Boolean
 
 from sympy.assumptions import ask, Q  # type: ignore
@@ -13,13 +15,13 @@ def refine(expr, assumptions=True):
     Explanation
     ===========
 
-    Unlike :func:`~.simplify()` which performs structural simplification
+    Unlike :func:`~.simplify` which performs structural simplification
     without any assumption, this function transforms the expression into
     the form which is only valid under certain assumptions. Note that
     ``simplify()`` is generally not done in refining process.
 
     Refining boolean expression involves reducing it to ``S.true`` or
-    ``S.false``. Unlike :func:`~.ask()`, the expression will not be reduced
+    ``S.false``. Unlike :func:`~.ask`, the expression will not be reduced
     if the truth value cannot be determined.
 
     Examples
@@ -83,8 +85,7 @@ def refine_abs(expr, assumptions):
     -x
 
     """
-    from sympy.core.logic import fuzzy_not
-    from sympy import Abs
+    from sympy.functions.elementary.complexes import Abs
     arg = expr.args[0]
     if ask(Q.real(arg), assumptions) and \
             fuzzy_not(ask(Q.negative(arg), assumptions)):
@@ -133,7 +134,6 @@ def refine_Pow(expr, assumptions):
     (-1)**(x + 1)
 
     """
-    from sympy.core import Pow, Rational
     from sympy.functions.elementary.complexes import Abs
     from sympy.functions import sign
     if isinstance(expr.base, Abs):
@@ -147,7 +147,7 @@ def refine_Pow(expr, assumptions):
             if ask(Q.odd(expr.exp), assumptions):
                 return sign(expr.base) * abs(expr.base) ** expr.exp
         if isinstance(expr.exp, Rational):
-            if type(expr.base) is Pow:
+            if isinstance(expr.base, Pow):
                 return abs(expr.base.base) ** (expr.base.exp * expr.exp)
 
         if expr.base is S.NegativeOne:
@@ -232,7 +232,6 @@ def refine_atan2(expr, assumptions):
     nan
     """
     from sympy.functions.elementary.trigonometric import atan
-    from sympy.core import S
     y, x = expr.args
     if ask(Q.real(y) & Q.positive(x), assumptions):
         return atan(y / x)
@@ -380,8 +379,7 @@ def refine_matrixelement(expr, assumptions):
     ========
 
     >>> from sympy.assumptions.refine import refine_matrixelement
-    >>> from sympy import Q
-    >>> from sympy.matrices.expressions.matexpr import MatrixSymbol
+    >>> from sympy import MatrixSymbol, Q
     >>> X = MatrixSymbol('X', 3, 3)
     >>> refine_matrixelement(X[0, 1], Q.symmetric(X))
     X[0, 1]
@@ -395,7 +393,7 @@ def refine_matrixelement(expr, assumptions):
             return expr
         return MatrixElement(matrix, j, i)
 
-handlers_dict = {
+handlers_dict: dict[str, Callable[[Expr, Boolean], Expr]] = {
     'Abs': refine_abs,
     'Pow': refine_Pow,
     'atan2': refine_atan2,
@@ -404,4 +402,4 @@ handlers_dict = {
     'arg': refine_arg,
     'sign': refine_sign,
     'MatrixElement': refine_matrixelement
-}  # type: Dict[str, Callable[[Expr, Boolean], Expr]]
+}
