@@ -85,12 +85,7 @@ def _min_dummies(dummies, sym, indices):
     [0, 1, 2, 2, 2, 2, 2, 2, 8, 9]
     """
     num_types = len(sym)
-    m = []
-    for dx in dummies:
-        if dx:
-            m.append(min(dx))
-        else:
-            m.append(None)
+    m = [min(dx) if dx else None for dx in dummies]
     res = indices[:]
     for i in range(num_types):
         for c, i in enumerate(indices):
@@ -162,7 +157,7 @@ def transversal2coset(size, base, transversal):
 
 
 def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
-    """
+    r"""
     Butler-Portugal algorithm for tensor canonicalization with dummy indices.
 
     Parameters
@@ -227,17 +222,17 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     (the indices including both the contravariant and the covariant ones)
     can be written as
 
-    `t = T(ind[g[0]],..., ind[g[n-1]])`,
+    `t = T(ind[g[0]], \dots, ind[g[n-1]])`,
 
-    where `n= len(ind)`;
+    where `n = len(ind)`;
     `g` has size `n + 2`, the last two indices for the sign of the tensor
     (trick introduced in [4]).
 
     A slot symmetry transformation `s` is a permutation acting on the slots
-    `t -> T(ind[(g*s)[0]],..., ind[(g*s)[n-1]])`
+    `t \rightarrow T(ind[(g*s)[0]], \dots, ind[(g*s)[n-1]])`
 
     A dummy symmetry transformation acts on `ind`
-    `t -> T(ind[(d*g)[0]],..., ind[(d*g)[n-1]])`
+    `t \rightarrow T(ind[(d*g)[0]], \dots, ind[(d*g)[n-1]])`
 
     Being interested only in the transformations of the tensor under
     these symmetries, one can represent the tensor by `g`, which transforms
@@ -283,7 +278,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     the dummy indices themselves (d1 <-> d2).
 
     The dummy symmetry acts from the left
-    `d = [1, 0, 2, 3, 4, 5, 6, 7]`  exchange `d1 <-> -d1`
+    `d = [1, 0, 2, 3, 4, 5, 6, 7]`  exchange `d1 \leftrightarrow -d1`
     `T^{d3 d2 d1}{}_{d1 d2 d3} == T^{d3 d2}{}_{d1}{}^{d1}{}_{d2 d3}`
 
     `g=[4, 2, 0, 1, 3, 5, 6, 7]  -> [4, 2, 1, 0, 3, 5, 6, 7] = _af_rmul(d, g)`
@@ -311,38 +306,38 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     from the sign, return zero; otherwise
     choose as representative the tensor with indices
     ordered lexicographically according to `[d1, -d1, d2, -d2, d3, -d3]`
-    that is `rep = min(D*g*S) = min([d*g*s for d in D for s in S])`
+    that is ``rep = min(D*g*S) = min([d*g*s for d in D for s in S])``
 
     The indices are fixed one by one; first choose the lowest index
     for slot 0, then the lowest remaining index for slot 1, etc.
     Doing this one obtains a chain of stabilizers
 
-    `S -> S_{b0} -> S_{b0,b1} -> ...` and
-    `D -> D_{p0} -> D_{p0,p1} -> ...`
+    `S \rightarrow S_{b0} \rightarrow S_{b0,b1} \rightarrow \dots` and
+    `D \rightarrow D_{p0} \rightarrow D_{p0,p1} \rightarrow \dots`
 
-    where `[b0, b1, ...] = range(b)` is a base of the symmetric group;
+    where ``[b0, b1, ...] = range(b)`` is a base of the symmetric group;
     the strong base `b_S` of S is an ordered sublist of it;
     therefore it is sufficient to compute once the
     strong base generators of S using the Schreier-Sims algorithm;
     the stabilizers of the strong base generators are the
     strong base generators of the stabilizer subgroup.
 
-    `dbase = [p0, p1, ...]` is not in general in lexicographic order,
+    ``dbase = [p0, p1, ...]`` is not in general in lexicographic order,
     so that one must recompute the strong base generators each time;
     however this is trivial, there is no need to use the Schreier-Sims
     algorithm for D.
 
     The algorithm keeps a TAB of elements `(s_i, d_i, h_i)`
-    where `h_i = d_i*g*s_i` satisfying `h_i[j] = p_j` for `0 <= j < i`
+    where `h_i = d_i \times g \times s_i` satisfying `h_i[j] = p_j` for `0 \le j < i`
     starting from `s_0 = id, d_0 = id, h_0 = g`.
 
-    The equations `h_0[0] = p_0, h_1[1] = p_1,...` are solved in this order,
+    The equations `h_0[0] = p_0, h_1[1] = p_1, \dots` are solved in this order,
     choosing each time the lowest possible value of p_i
 
     For `j < i`
-    `d_i*g*s_i*S_{b_0,...,b_{i-1}}*b_j = D_{p_0,...,p_{i-1}}*p_j`
-    so that for dx in `D_{p_0,...,p_{i-1}}` and sx in
-    `S_{base[0],...,base[i-1]}` one has `dx*d_i*g*s_i*sx*b_j = p_j`
+    `d_i*g*s_i*S_{b_0, \dots, b_{i-1}}*b_j = D_{p_0, \dots, p_{i-1}}*p_j`
+    so that for dx in `D_{p_0,\dots,p_{i-1}}` and sx in
+    `S_{base[0], \dots, base[i-1]}` one has `dx*d_i*g*s_i*sx*b_j = p_j`
 
     Search for dx, sx such that this equation holds for `j = i`;
     it can be written as `s_i*sx*b_j = J, dx*d_i*g*J = p_j`
@@ -403,7 +398,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     g = g.array_form
     num_dummies = size - 2
     indices = list(range(num_dummies))
-    all_metrics_with_sym = all([_ is not None for _ in sym])
+    all_metrics_with_sym = not any(_ is None for _ in sym)
     num_types = len(sym)
     dumx = dummies[:]
     dumx_flat = []
@@ -436,7 +431,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
             md = [min(_orbit(size, [_af_new(
                 ddx) for ddx in dsgsx], ii)) for ii in range(size - 2)]
 
-        p_i = min([min([md[h[x]] for x in deltab]) for s, d, h in TAB])
+        p_i = min(min(md[h[x]] for x in deltab) for s, d, h in TAB)
         dsgsx1 = [_af_new(_) for _ in dsgsx]
         Dxtrav = _orbit_transversal(size, dsgsx1, p_i, False, af=True) \
             if dsgsx else None
@@ -459,7 +454,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
         TAB1 = []
         while TAB:
             s, d, h = TAB.pop()
-            if min([md[h[x]] for x in deltab]) != p_i:
+            if min(md[h[x]] for x in deltab) != p_i:
                 continue
             deltab1 = [x for x in deltab if md[h[x]] == p_i]
             # NEXT = s*deltab1 intersection (d*g)**-1*deltap
@@ -593,7 +588,7 @@ def canonical_free(base, gens, g, num_free):
         if x not in base:
             base.append(x)
     h = g
-    for i, transv in enumerate(transversals):
+    for transv in transversals:
         h_i = [size]*num_free
         # find the element s in transversals[i] such that
         # _af_rmul(h, s) has its free elements with the lowest position in h
@@ -763,12 +758,12 @@ def canonicalize(g, dummies, msym, *v):
     """
     from sympy.combinatorics.testutil import canonicalize_naive
     if not isinstance(msym, list):
-        if not msym in [0, 1, None]:
+        if msym not in (0, 1, None):
             raise ValueError('msym must be 0, 1 or None')
         num_types = 1
     else:
         num_types = len(msym)
-        if not all(msymx in [0, 1, None] for msymx in msym):
+        if not all(msymx in (0, 1, None) for msymx in msym):
             raise ValueError('msym entries must be 0, 1 or None')
         if len(dummies) != num_types:
             raise ValueError(
@@ -776,8 +771,7 @@ def canonicalize(g, dummies, msym, *v):
     size = g.size
     num_tensors = 0
     v1 = []
-    for i in range(len(v)):
-        base_i, gens_i, n_i, sym_i = v[i]
+    for base_i, gens_i, n_i, sym_i in v:
         # check that the BSGS is minimal;
         # this property is used in double_coset_can_rep;
         # if it is not minimal use canonicalize_naive
@@ -819,9 +813,8 @@ def canonicalize(g, dummies, msym, *v):
     # Determine free_i, the list of slots of tensors which are fixed
     # since they are occupied by free indices, which are fixed.
     start = 0
-    for i in range(len(v)):
+    for i, (base_i, gens_i, n_i, sym_i) in enumerate(v):
         free_i = []
-        base_i, gens_i, n_i, sym_i = v[i]
         len_tens = gens_i[0].size - 2
         # for each component tensor get a list od fixed islots
         for j in range(n_i):
@@ -945,8 +938,8 @@ def get_symmetric_group_sgs(n, antisym=False):
     Parameters
     ==========
 
-    ``n``: rank of the tensor
-    ``antisym`` : bool
+    n : rank of the tensor
+    antisym : bool
         ``antisym = False`` symmetric tensor
         ``antisym = True``  antisymmetric tensor
 
