@@ -1,8 +1,9 @@
 """sympify -- convert objects SymPy internal format"""
 
-import typing
-if typing.TYPE_CHECKING:
-    from typing import Any, Callable, Dict as tDict, Type
+from __future__ import annotations
+from typing import Any, Callable
+
+import mpmath.libmp as mlib
 
 from inspect import getmro
 import string
@@ -10,7 +11,6 @@ from sympy.core.random import choice
 
 from .parameters import global_parameters
 
-from sympy.utilities.exceptions import sympy_deprecation_warning
 from sympy.utilities.iterables import iterable
 
 
@@ -28,10 +28,10 @@ class SympifyError(ValueError):
             str(self.base_exc)))
 
 
-converter = {}  # type: tDict[Type[Any], Callable[[Any], Basic]]
+converter: dict[type[Any], Callable[[Any], Basic]] = {}
 
 #holds the conversions defined in SymPy itself, i.e. non-user defined conversions
-_sympy_converter = {} # type: tDict[Type[Any], Callable[[Any], Basic]]
+_sympy_converter: dict[type[Any], Callable[[Any], Basic]] = {}
 
 #alias for clearer use in the library
 _external_converter = converter
@@ -86,17 +86,13 @@ def _convert_numpy_types(a, **sympify_args):
         else:
             return sympify(a.item(), **sympify_args)
     else:
-        try:
-            from .numbers import Float
-            prec = np.finfo(a).nmant + 1
-            # E.g. double precision means prec=53 but nmant=52
-            # Leading bit of mantissa is always 1, so is not stored
-            a = str(list(np.reshape(np.asarray(a),
-                                    (1, np.size(a)))[0]))[1:-1]
-            return Float(a, precision=prec)
-        except NotImplementedError:
-            raise SympifyError('Translation for numpy float : %s '
-                               'is not implemented' % a)
+        from .numbers import Float
+        prec = np.finfo(a).nmant + 1
+        # E.g. double precision means prec=53 but nmant=52
+        # Leading bit of mantissa is always 1, so is not stored
+        p, q = a.as_integer_ratio()
+        a = mlib.from_rational(p, q, prec)
+        return Float(a, precision=prec)
 
 
 def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
@@ -144,6 +140,22 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     ...
     SympifyError: SympifyError: "could not parse 'x***2'"
 
+    When attempting to parse non-Python syntax using ``sympify``, it raises a
+    ``SympifyError``:
+
+    >>> sympify("2x+1")
+    Traceback (most recent call last):
+    ...
+    SympifyError: Sympify of expression 'could not parse '2x+1'' failed
+
+    To parse non-Python syntax, use ``parse_expr`` from ``sympy.parsing.sympy_parser``.
+
+    >>> from sympy.parsing.sympy_parser import parse_expr
+    >>> parse_expr("2x+1", transformations="all")
+    2*x + 1
+
+    For more details about ``transformations``: see :func:`~sympy.parsing.sympy_parser.parse_expr`
+
     Locals
     ------
 
@@ -190,7 +202,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     multi-letter names that are defined in ``abc``).
 
     >>> from sympy.abc import _clash1
-    >>> set(_clash1)
+    >>> set(_clash1)  # if this fails, see issue #23903
     {'E', 'I', 'N', 'O', 'Q', 'S'}
     >>> sympify('I & Q', _clash1)
     I & Q
@@ -402,13 +414,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
 
     _sympy_ = getattr(a, "_sympy_", None)
     if _sympy_ is not None:
-        try:
-            return a._sympy_()
-        # XXX: Catches AttributeError: 'SymPyConverter' object has no
-        # attribute 'tuple'
-        # This is probably a bug somewhere but for now we catch it here.
-        except AttributeError:
-            pass
+        return a._sympy_()
 
     if not strict:
         # Put numpy array conversion _before_ float/int, see
@@ -437,14 +443,12 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
                                        evaluate=evaluate)
                     except SympifyError:
                         pass
-        else:
+        elif hasattr(a, '__float__'):
             # float and int can coerce size-one numpy arrays to their lone
             # element.  See issue https://github.com/numpy/numpy/issues/10404.
-            for coerce in (float, int):
-                try:
-                    return sympify(coerce(a))
-                except (TypeError, ValueError, AttributeError, SympifyError):
-                    continue
+            return sympify(float(a))
+        elif hasattr(a, '__int__'):
+            return sympify(int(a))
 
     if strict:
         raise SympifyError(a)
@@ -458,26 +462,7 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
             pass
 
     if not isinstance(a, str):
-        try:
-            a = str(a)
-        except Exception as exc:
-            raise SympifyError(a, exc)
-        sympy_deprecation_warning(
-            f"""
-The string fallback in sympify() is deprecated.
-
-To explicitly convert the string form of an object, use
-sympify(str(obj)). To add define sympify behavior on custom
-objects, use sympy.core.sympify.converter or define obj._sympy_
-(see the sympify() docstring).
-
-sympify() performed the string fallback resulting in the following string:
-
-{a!r}
-            """,
-            deprecated_since_version='1.6',
-            active_deprecations_target="deprecated-sympify-string-fallback",
-        )
+        raise SympifyError('cannot sympify object of type %r' % type(a))
 
     from sympy.parsing.sympy_parser import (parse_expr, TokenError,
                                             standard_transformations)
