@@ -13,7 +13,8 @@ from sympy.functions.elementary.hyperbolic import cosh, sinh, tanh
 from sympy.polys import Poly
 from sympy.series.order import O
 from sympy.sets import FiniteSet
-from sympy.core.power import power, integer_nthroot
+from sympy.core.power import power
+from sympy.core.intfunc import integer_nthroot
 from sympy.testing.pytest import warns, _both_exp_pow
 from sympy.utilities.exceptions import SymPyDeprecationWarning
 from sympy.abc import a, b, c, x, y
@@ -554,7 +555,7 @@ def test_issue_14815():
     assert sqrt(x).is_extended_negative is False
     x = Symbol('x', extended_real=True)
     assert sqrt(x).is_extended_negative is False
-    assert sqrt(zoo, evaluate=False).is_extended_negative is None
+    assert sqrt(zoo, evaluate=False).is_extended_negative is False
     assert sqrt(nan, evaluate=False).is_extended_negative is None
 
 
@@ -650,3 +651,13 @@ def test_powers_of_I():
 def test_issue_23918():
     b = S(2)/3
     assert (b**x).as_base_exp() == (1/b, -x)
+
+
+def test_issue_26546():
+    x = Symbol('x', real=True)
+    assert x.is_extended_real is True
+    assert sqrt(x+I).is_extended_real is False
+    assert Pow(x+I, S.Half).is_extended_real is False
+    assert Pow(x+I, Rational(1,2)).is_extended_real is False
+    assert Pow(x+I, Rational(1,13)).is_extended_real is False
+    assert Pow(x+I, Rational(2,3)).is_extended_real is None
