@@ -1,11 +1,27 @@
 from sympy.core.logic import fuzzy_and
 from sympy.core.sympify import _sympify
 from sympy.multipledispatch import dispatch
-from sympy.testing.pytest import XFAIL, raises, warns_deprecated_sympy
-from sympy import (S, Symbol, symbols, nan, oo, I, pi, Float, And, Or,
-                   Not, Implies, Xor, zoo, sqrt, Rational, simplify, Function,
-                   log, cos, sin, Add, Mul, Pow, floor, ceiling, trigsimp, Reals,
-                   Basic, Expr, Q, exp, exp_polar)
+from sympy.testing.pytest import XFAIL, raises
+from sympy.assumptions.ask import Q
+from sympy.core.add import Add
+from sympy.core.basic import Basic
+from sympy.core.expr import Expr, unchanged
+from sympy.core.function import Function
+from sympy.core.mul import Mul
+from sympy.core.numbers import (Float, I, Rational, nan, oo, pi, zoo)
+from sympy.core.power import Pow
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, symbols)
+from sympy.functions.elementary.complexes import sign, Abs
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.functions.elementary.exponential import (exp, exp_polar, log)
+from sympy.functions.elementary.integers import (ceiling, floor)
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.logic.boolalg import (And, Implies, Not, Or, Xor)
+from sympy.sets import Reals
+from sympy.simplify.simplify import simplify
+from sympy.simplify.trigsimp import trigsimp
 from sympy.core.relational import (Relational, Equality, Unequality,
                                    GreaterThan, LessThan, StrictGreaterThan,
                                    StrictLessThan, Rel, Eq, Lt, Le,
@@ -122,9 +138,6 @@ def test_Eq_Ne():
 
     assert Eq(x, x)  # issue 5719
 
-    with warns_deprecated_sympy():
-        assert Eq(x) == Eq(x, 0)
-
     # issue 6116
     p = Symbol('p', positive=True)
     assert Eq(p, 0) is S.false
@@ -263,7 +276,7 @@ def test_rich_cmp():
 
 
 def test_doit():
-    from sympy import Symbol
+    from sympy.core.symbol import Symbol
     p = Symbol('p', positive=True)
     n = Symbol('n', negative=True)
     np = Symbol('np', nonpositive=True)
@@ -365,7 +378,7 @@ def test_new_relational():
     assert (x < 0) != StrictLessThan(x, 1)
 
     # finally, some fuzz testing
-    from random import randint
+    from sympy.core.random import randint
     for i in range(100):
         while 1:
             strtype, length = (chr, 65535) if randint(0, 1) else (chr, 255)
@@ -670,8 +683,8 @@ def test_issue_8245():
     assert rel_check(a, a.n(10))
     assert rel_check(a, a.n(20))
     assert rel_check(a, a.n())
-    # prec of 30 is enough to fully capture a as mpf
-    assert Float(a, 30) == Float(str(a.p), '')/Float(str(a.q), '')
+    # prec of 31 is enough to fully capture a as mpf
+    assert Float(a, 31) == Float(str(a.p), '')/Float(str(a.q), '')
     for i in range(31):
         r = Rational(Float(a, i))
         f = Float(r)
@@ -787,10 +800,10 @@ def test_simplify_relational():
     assert Lt(x, 2).simplify() == Lt(x, 2)
     assert Lt(-x, 2).simplify() == Gt(x, -2)
 
-    # Test particulat branches of _eval_simplify
+    # Test particular branches of _eval_simplify
     m = exp(1) - exp_polar(1)
     assert simplify(m*x > 1) is S.false
-    # These two tests the same branch
+    # These two test the same branch
     assert simplify(m*x + 2*m*y > 1) is S.false
     assert simplify(m*x + y > 1 + y) is S.false
 
@@ -840,6 +853,7 @@ def test_canonical():
     assert [i.canonical for i in c] == c
     assert [i.reversed.canonical for i in c] == c
     assert not any(i.lhs.is_Number and not i.rhs.is_Number for i in c)
+    assert Eq(y < x, x > y).canonical is S.true
 
 
 @XFAIL
@@ -1014,15 +1028,16 @@ def test_rel_args():
                 raises(TypeError, lambda: Relational(b, v, op))
 
 
-def test_Equality_rewrite_as_Add():
-    eq = Eq(x + y, y - x)
-    assert eq.rewrite(Add) == 2*x
-    assert eq.rewrite(Add, evaluate=None).args == (x, x, y, -y)
-    assert eq.rewrite(Add, evaluate=False).args == (x, y, x, -y)
+def test_nothing_happens_to_Eq_condition_during_simplify():
+    # issue 25701
+    r = symbols('r', real=True)
+    assert Eq(2*sign(r + 3)/(5*Abs(r + 3)**Rational(3, 5)), 0
+        ).simplify() == Eq(Piecewise(
+        (0, Eq(r, -3)), ((r + 3)/(5*Abs((r + 3)**Rational(8, 5)))*2, True)), 0)
 
 
 def test_issue_15847():
-    a = Ne(x*(x+y), x**2 + x*y)
+    a = Ne(x*(x + y), x**2 + x*y)
     assert simplify(a) == False
 
 
@@ -1153,7 +1168,7 @@ def test_EvalEq():
     This test exists to ensure backwards compatibility.
     The method to use is _eval_is_eq
     """
-    from sympy import Expr
+    from sympy.core.expr import Expr
 
     class PowTest(Expr):
         def __new__(cls, base, exp):
@@ -1218,3 +1233,39 @@ def test_is_ge_le():
     assert is_gt(PowTest(3, 9), PowTest(3,2))
     assert is_le(PowTest(3, 2), PowTest(3,9))
     assert is_lt(PowTest(3, 2), PowTest(3,9))
+
+
+def test_weak_strict():
+    for func in (Eq, Ne):
+        eq = func(x, 1)
+        assert eq.strict == eq.weak == eq
+    eq = Gt(x, 1)
+    assert eq.weak == Ge(x, 1)
+    assert eq.strict == eq
+    eq = Lt(x, 1)
+    assert eq.weak == Le(x, 1)
+    assert eq.strict == eq
+    eq = Ge(x, 1)
+    assert eq.strict == Gt(x, 1)
+    assert eq.weak == eq
+    eq = Le(x, 1)
+    assert eq.strict == Lt(x, 1)
+    assert eq.weak == eq
+
+
+def test_issue_23731():
+    i = symbols('i', integer=True)
+    assert unchanged(Eq, i, 1.0)
+    assert unchanged(Eq, i/2, 0.5)
+    ni = symbols('ni', integer=False)
+    assert Eq(ni, 1) == False
+    assert unchanged(Eq, ni, .1)
+    assert Eq(ni, 1.0) == False
+    nr = symbols('nr', rational=False)
+    assert Eq(nr, .1) == False
+
+
+def test_rewrite_Add():
+    from sympy.testing.pytest import warns_deprecated_sympy
+    with warns_deprecated_sympy():
+        assert Eq(x, y).rewrite(Add) == x - y
