@@ -1,16 +1,16 @@
+from typing import Tuple as tTuple
+
 from sympy.core import S, Add, Mul, sympify, Symbol, Dummy, Basic
 from sympy.core.expr import Expr
 from sympy.core.exprtools import factor_terms
 from sympy.core.function import (Function, Derivative, ArgumentIndexError,
-    AppliedUndef)
+    AppliedUndef, expand_mul)
 from sympy.core.logic import fuzzy_not, fuzzy_or
 from sympy.core.numbers import pi, I, oo
+from sympy.core.power import Pow
 from sympy.core.relational import Eq
-from sympy.functions.elementary.exponential import exp, exp_polar, log
-from sympy.functions.elementary.integers import ceiling
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.functions.elementary.piecewise import Piecewise
-from sympy.functions.elementary.trigonometric import atan, atan2
 
 ###############################################################################
 ######################### REAL and IMAGINARY PARTS ############################
@@ -22,7 +22,7 @@ class re(Function):
     Returns real part of expression. This function performs only
     elementary analysis and so it will fail to decompose properly
     more complicated expressions. If completely simplified result
-    is needed then use Basic.as_real_imag() or perform complex
+    is needed then use ``Basic.as_real_imag()`` or perform complex
     expansion on instance of this function.
 
     Examples
@@ -59,6 +59,8 @@ class re(Function):
     im
     """
 
+    args: tTuple[Expr]
+
     is_extended_real = True
     unbranched = True  # implicitly works on the projection to C
     _singularities = True  # non-holomorphic
@@ -71,7 +73,7 @@ class re(Function):
             return S.NaN
         elif arg.is_extended_real:
             return arg
-        elif arg.is_imaginary or (S.ImaginaryUnit*arg).is_extended_real:
+        elif arg.is_imaginary or (I*arg).is_extended_real:
             return S.Zero
         elif arg.is_Matrix:
             return arg.as_real_imag()[0]
@@ -82,12 +84,12 @@ class re(Function):
             included, reverted, excluded = [], [], []
             args = Add.make_args(arg)
             for term in args:
-                coeff = term.as_coefficient(S.ImaginaryUnit)
+                coeff = term.as_coefficient(I)
 
                 if coeff is not None:
                     if not coeff.is_extended_real:
                         reverted.append(coeff)
-                elif not term.has(S.ImaginaryUnit) and term.is_extended_real:
+                elif not term.has(I) and term.is_extended_real:
                     excluded.append(term)
                 else:
                     # Try to do some advanced expansion.  If
@@ -115,11 +117,11 @@ class re(Function):
         if x.is_extended_real or self.args[0].is_extended_real:
             return re(Derivative(self.args[0], x, evaluate=True))
         if x.is_imaginary or self.args[0].is_imaginary:
-            return -S.ImaginaryUnit \
+            return -I \
                 * im(Derivative(self.args[0], x, evaluate=True))
 
     def _eval_rewrite_as_im(self, arg, **kwargs):
-        return self.args[0] - S.ImaginaryUnit*im(self.args[0])
+        return self.args[0] - I*im(self.args[0])
 
     def _eval_is_algebraic(self):
         return self.args[0].is_algebraic
@@ -142,7 +144,7 @@ class im(Function):
     Returns imaginary part of expression. This function performs only
     elementary analysis and so it will fail to decompose properly more
     complicated expressions. If completely simplified result is needed then
-    use Basic.as_real_imag() or perform complex expansion on instance of
+    use ``Basic.as_real_imag()`` or perform complex expansion on instance of
     this function.
 
     Examples
@@ -179,6 +181,8 @@ class im(Function):
     re
     """
 
+    args: tTuple[Expr]
+
     is_extended_real = True
     unbranched = True  # implicitly works on the projection to C
     _singularities = True  # non-holomorphic
@@ -191,8 +195,8 @@ class im(Function):
             return S.NaN
         elif arg.is_extended_real:
             return S.Zero
-        elif arg.is_imaginary or (S.ImaginaryUnit*arg).is_extended_real:
-            return -S.ImaginaryUnit * arg
+        elif arg.is_imaginary or (I*arg).is_extended_real:
+            return -I * arg
         elif arg.is_Matrix:
             return arg.as_real_imag()[1]
         elif arg.is_Function and isinstance(arg, conjugate):
@@ -201,14 +205,14 @@ class im(Function):
             included, reverted, excluded = [], [], []
             args = Add.make_args(arg)
             for term in args:
-                coeff = term.as_coefficient(S.ImaginaryUnit)
+                coeff = term.as_coefficient(I)
 
                 if coeff is not None:
                     if not coeff.is_extended_real:
                         reverted.append(coeff)
                     else:
                         excluded.append(coeff)
-                elif term.has(S.ImaginaryUnit) or not term.is_extended_real:
+                elif term.has(I) or not term.is_extended_real:
                     # Try to do some advanced expansion.  If
                     # impossible, don't try to do im(arg) again
                     # (because this is what we are trying to do now).
@@ -234,11 +238,11 @@ class im(Function):
         if x.is_extended_real or self.args[0].is_extended_real:
             return im(Derivative(self.args[0], x, evaluate=True))
         if x.is_imaginary or self.args[0].is_imaginary:
-            return -S.ImaginaryUnit \
+            return -I \
                 * re(Derivative(self.args[0], x, evaluate=True))
 
     def _eval_rewrite_as_re(self, arg, **kwargs):
-        return -S.ImaginaryUnit*(self.args[0] - re(self.args[0]))
+        return -I*(self.args[0] - re(self.args[0]))
 
     def _eval_is_algebraic(self):
         return self.args[0].is_algebraic
@@ -267,14 +271,14 @@ class sign(Function):
 
     If the expression is real the sign will be:
 
-        * 1 if expression is positive
-        * 0 if expression is equal to zero
-        * -1 if expression is negative
+        * $1$ if expression is positive
+        * $0$ if expression is equal to zero
+        * $-1$ if expression is negative
 
     If the expression is imaginary the sign will be:
 
-        * I if im(expression) is positive
-        * -I if im(expression) is negative
+        * $I$ if im(expression) is positive
+        * $-I$ if im(expression) is negative
 
     Otherwise an unevaluated expression will be returned. When evaluated, the
     result (in general) will be ``cos(arg(expr)) + I*sin(arg(expr))``.
@@ -282,8 +286,7 @@ class sign(Function):
     Examples
     ========
 
-    >>> from sympy.functions import sign
-    >>> from sympy.core.numbers import I
+    >>> from sympy import sign, I
 
     >>> sign(-1)
     -1
@@ -339,7 +342,7 @@ class sign(Function):
                     if a.is_imaginary:
                         ai = im(a)
                         if ai.is_comparable:  # i.e. a = I*real
-                            s *= S.ImaginaryUnit
+                            s *= I
                             if ai.is_extended_negative:
                                 # can't use sign(ai) here since ai might not be
                                 # a Number
@@ -366,12 +369,12 @@ class sign(Function):
             if arg.is_Pow and arg.exp is S.Half:
                 # we catch this because non-trivial sqrt args are not expanded
                 # e.g. sqrt(1-sqrt(2)) --x-->  to I*sqrt(sqrt(2) - 1)
-                return S.ImaginaryUnit
-            arg2 = -S.ImaginaryUnit * arg
+                return I
+            arg2 = -I * arg
             if arg2.is_extended_positive:
-                return S.ImaginaryUnit
+                return I
             if arg2.is_extended_negative:
-                return -S.ImaginaryUnit
+                return -I
 
     def _eval_Abs(self):
         if fuzzy_not(self.args[0].is_zero):
@@ -388,7 +391,7 @@ class sign(Function):
         elif self.args[0].is_imaginary:
             from sympy.functions.special.delta_functions import DiracDelta
             return 2 * Derivative(self.args[0], x, evaluate=True) \
-                * DiracDelta(-S.ImaginaryUnit * self.args[0])
+                * DiracDelta(-I * self.args[0])
 
     def _eval_is_nonnegative(self):
         if self.args[0].is_nonnegative:
@@ -447,9 +450,9 @@ class Abs(Function):
     Explanation
     ===========
 
-    This is an extension of the built-in function abs() to accept symbolic
-    values.  If you pass a SymPy expression to the built-in abs(), it will
-    pass it automatically to Abs().
+    This is an extension of the built-in function ``abs()`` to accept symbolic
+    values.  If you pass a SymPy expression to the built-in ``abs()``, it will
+    pass it automatically to ``Abs()``.
 
     Examples
     ========
@@ -477,7 +480,7 @@ class Abs(Function):
         >>> type(abs(S.NegativeOne))
         <class 'sympy.core.numbers.One'>
 
-    Abs will always return a sympy object.
+    Abs will always return a SymPy object.
 
     Parameters
     ==========
@@ -498,6 +501,8 @@ class Abs(Function):
     sign, conjugate
     """
 
+    args: tTuple[Expr]
+
     is_extended_real = True
     is_extended_negative = False
     is_extended_nonnegative = True
@@ -517,8 +522,6 @@ class Abs(Function):
     @classmethod
     def eval(cls, arg):
         from sympy.simplify.simplify import signsimp
-        from sympy.core.function import expand_mul
-        from sympy.core.power import Pow
 
         if hasattr(arg, '_eval_Abs'):
             obj = arg._eval_Abs()
@@ -555,7 +558,9 @@ class Abs(Function):
         if arg is S.NaN:
             return S.NaN
         if arg is S.ComplexInfinity:
-            return S.Infinity
+            return oo
+        from sympy.functions.elementary.exponential import exp, log
+
         if arg.is_Pow:
             base, exponent = arg.as_base_exp()
             if base.is_extended_real:
@@ -568,7 +573,7 @@ class Abs(Function):
                 if base.is_extended_nonnegative:
                     return base**re(exponent)
                 if base.is_extended_negative:
-                    return (-base)**re(exponent)*exp(-S.Pi*im(exponent))
+                    return (-base)**re(exponent)*exp(-pi*im(exponent))
                 return
             elif not base.has(Symbol): # complex base
                 # express base**exponent as exp(exponent*log(base))
@@ -583,9 +588,9 @@ class Abs(Function):
             elif arg.is_negative:
                 return -arg
             return
-        if arg.is_Add and arg.has(S.Infinity, S.NegativeInfinity):
+        if arg.is_Add and arg.has(oo, S.NegativeInfinity):
             if any(a.is_infinite for a in arg.as_real_imag()):
-                return S.Infinity
+                return oo
         if arg.is_zero:
             return S.Zero
         if arg.is_extended_nonnegative:
@@ -593,7 +598,7 @@ class Abs(Function):
         if arg.is_extended_nonpositive:
             return -arg
         if arg.is_imaginary:
-            arg2 = -S.ImaginaryUnit * arg
+            arg2 = -I * arg
             if arg2.is_extended_nonnegative:
                 return arg2
         if arg.is_extended_real:
@@ -626,9 +631,7 @@ class Abs(Function):
         return self._args[0].is_zero
 
     def _eval_is_extended_positive(self):
-        is_z = self.is_zero
-        if is_z is not None:
-            return not is_z
+        return fuzzy_not(self._args[0].is_zero)
 
     def _eval_is_rational(self):
         if self.args[0].is_extended_real:
@@ -654,6 +657,7 @@ class Abs(Function):
         return
 
     def _eval_nseries(self, x, n, logx, cdir=0):
+        from sympy.functions.elementary.exponential import log
         direction = self.args[0].leadterm(x)[0]
         if direction.has(log(x)):
             direction = direction.subs(log(x), logx)
@@ -686,21 +690,25 @@ class Abs(Function):
         return arg/sign(arg)
 
     def _eval_rewrite_as_conjugate(self, arg, **kwargs):
-        return (arg*conjugate(arg))**S.Half
+        return sqrt(arg*conjugate(arg))
 
 
 class arg(Function):
-    """
-    returns the argument (in radians) of a complex number.  The argument is
-    evaluated in consistent convention with atan2 where the branch-cut is
-    taken along the negative real axis and arg(z) is in the interval
-    (-pi,pi].  For a positive number, the argument is always 0.
+    r"""
+    Returns the argument (in radians) of a complex number. The argument is
+    evaluated in consistent convention with ``atan2`` where the branch-cut is
+    taken along the negative real axis and ``arg(z)`` is in the interval
+    $(-\pi,\pi]$. For a positive number, the argument is always 0; the
+    argument of a negative number is $\pi$; and the argument of 0
+    is undefined and returns ``nan``. So the ``arg`` function will never nest
+    greater than 3 levels since at the 4th application, the result must be
+    nan; for a real number, nan is returned on the 3rd application.
 
     Examples
     ========
 
-    >>> from sympy.functions import arg
-    >>> from sympy import I, sqrt
+    >>> from sympy import arg, I, sqrt, Dummy
+    >>> from sympy.abc import x
     >>> arg(2.0)
     0
     >>> arg(I)
@@ -713,6 +721,11 @@ class arg(Function):
     atan(3/4)
     >>> arg(0.8 + 0.6*I)
     0.643501108793284
+    >>> arg(arg(arg(arg(x))))
+    nan
+    >>> real = Dummy(real=True)
+    >>> arg(arg(arg(real)))
+    nan
 
     Parameters
     ==========
@@ -735,8 +748,27 @@ class arg(Function):
 
     @classmethod
     def eval(cls, arg):
+        a = arg
+        for i in range(3):
+            if isinstance(a, cls):
+                a = a.args[0]
+            else:
+                if i == 2 and a.is_extended_real:
+                    return S.NaN
+                break
+        else:
+            return S.NaN
+        from sympy.functions.elementary.exponential import exp, exp_polar
         if isinstance(arg, exp_polar):
             return periodic_argument(arg, oo)
+        elif isinstance(arg, exp):
+            i_ = im(arg.args[0])
+            if i_.is_comparable:
+                i_ %= 2*S.Pi
+                if i_ > S.Pi:
+                    i_ -= 2*S.Pi
+                return i_
+
         if not arg.is_Atom:
             c, arg_ = factor_terms(arg).as_coeff_Mul()
             if arg_.is_Mul:
@@ -747,6 +779,7 @@ class arg(Function):
             arg_ = arg
         if any(i.is_extended_positive is None for i in arg_.atoms(AppliedUndef)):
             return
+        from sympy.functions.elementary.trigonometric import atan2
         x, y = arg_.as_real_imag()
         rv = atan2(y, x)
         if rv.is_number:
@@ -760,18 +793,19 @@ class arg(Function):
                     Derivative(x, t, evaluate=True)) / (x**2 + y**2)
 
     def _eval_rewrite_as_atan2(self, arg, **kwargs):
+        from sympy.functions.elementary.trigonometric import atan2
         x, y = self.args[0].as_real_imag()
         return atan2(y, x)
 
 
 class conjugate(Function):
     """
-    Returns the `complex conjugate` Ref[1] of an argument.
+    Returns the *complex conjugate* [1]_ of an argument.
     In mathematics, the complex conjugate of a complex number
     is given by changing the sign of the imaginary part.
 
     Thus, the conjugate of the complex number
-    :math:`a + ib` (where a and b are real numbers) is :math:`a - ib`
+    :math:`a + ib` (where $a$ and $b$ are real numbers) is :math:`a - ib`
 
     Examples
     ========
@@ -816,6 +850,9 @@ class conjugate(Function):
         if obj is not None:
             return obj
 
+    def inverse(self):
+        return conjugate
+
     def _eval_Abs(self):
         return Abs(self.args[0], evaluate=True)
 
@@ -845,9 +882,7 @@ class transpose(Function):
     Examples
     ========
 
-    >>> from sympy.functions import transpose
-    >>> from sympy.matrices import MatrixSymbol
-    >>> from sympy import Matrix
+    >>> from sympy import transpose, Matrix, MatrixSymbol
     >>> A = MatrixSymbol('A', 25, 9)
     >>> transpose(A)
     A.T
@@ -904,8 +939,7 @@ class adjoint(Function):
     Examples
     ========
 
-    >>> from sympy import adjoint
-    >>> from sympy.matrices import MatrixSymbol
+    >>> from sympy import adjoint, MatrixSymbol
     >>> A = MatrixSymbol('A', 10, 5)
     >>> adjoint(A)
     Adjoint(A)
@@ -1015,6 +1049,7 @@ class polar_lift(Function):
             # but for now we will just be more restrictive and
             # see that it has evaluated to one of the known values.
             if ar in (0, pi/2, -pi/2, pi):
+                from sympy.functions.elementary.exponential import exp_polar
                 return exp_polar(I*ar)*abs(arg)
 
         if arg.is_Mul:
@@ -1037,6 +1072,7 @@ class polar_lift(Function):
             elif included:
                 return Mul(*(included + positive))
             else:
+                from sympy.functions.elementary.exponential import exp_polar
                 return Mul(*positive)*exp_polar(0)
 
     def _eval_evalf(self, prec):
@@ -1048,10 +1084,10 @@ class polar_lift(Function):
 
 
 class periodic_argument(Function):
-    """
+    r"""
     Represent the argument on a quotient of the Riemann surface of the
     logarithm. That is, given a period $P$, always return a value in
-    (-P/2, P/2], by using exp(P*I) == 1.
+    $(-P/2, P/2]$, by using $\exp(PI) = 1$.
 
     Examples
     ========
@@ -1077,7 +1113,7 @@ class periodic_argument(Function):
     ar : Expr
         A polar number.
 
-    period : ExprT
+    period : Expr
         The period $P$.
 
     See Also
@@ -1090,6 +1126,7 @@ class periodic_argument(Function):
 
     @classmethod
     def _getunbranched(cls, ar):
+        from sympy.functions.elementary.exponential import exp_polar, log
         if ar.is_Mul:
             args = ar.args
         else:
@@ -1129,11 +1166,13 @@ class periodic_argument(Function):
         unbranched = cls._getunbranched(ar)
         if unbranched is None:
             return None
+        from sympy.functions.elementary.trigonometric import atan, atan2
         if unbranched.has(periodic_argument, atan2, atan):
             return None
         if period == oo:
             return unbranched
         if period != oo:
+            from sympy.functions.elementary.integers import ceiling
             n = ceiling(unbranched/period - S.Half)*period
             if not n.has(ceiling):
                 return unbranched - n
@@ -1146,6 +1185,7 @@ class periodic_argument(Function):
                 return self
             return unbranched._eval_evalf(prec)
         ub = periodic_argument(z, oo)._eval_evalf(prec)
+        from sympy.functions.elementary.integers import ceiling
         return (ub - ceiling(ub/period - S.Half)*period)._eval_evalf(prec)
 
 
@@ -1181,7 +1221,7 @@ class principal_branch(Function):
 
     This is a function of two arguments. The first argument is a polar
     number `z`, and the second one a positive real number or infinity, `p`.
-    The result is "z mod exp_polar(I*p)".
+    The result is ``z mod exp_polar(I*p)``.
 
     Examples
     ========
@@ -1217,7 +1257,7 @@ class principal_branch(Function):
 
     @classmethod
     def eval(self, x, period):
-        from sympy import oo, exp_polar, I, Mul, polar_lift, Symbol
+        from sympy.functions.elementary.exponential import exp_polar
         if isinstance(x, polar_lift):
             return principal_branch(x.args[0], period)
         if period == oo:
@@ -1268,16 +1308,16 @@ class principal_branch(Function):
             return exp_polar(arg*I)*abs(c)
 
     def _eval_evalf(self, prec):
-        from sympy import exp, pi, I
         z, period = self.args
         p = periodic_argument(z, period)._eval_evalf(prec)
         if abs(p) > pi or p == -pi:
             return self  # Cannot evalf for this argument.
+        from sympy.functions.elementary.exponential import exp
         return (abs(z)*exp(I*p))._eval_evalf(prec)
 
 
 def _polarify(eq, lift, pause=False):
-    from sympy import Integral
+    from sympy.integrals.integrals import Integral
     if eq.is_polar:
         return eq
     if eq.is_number and not pause:
@@ -1315,17 +1355,17 @@ def polarify(eq, subs=True, lift=False):
     choice of argument).
 
     Note that no attempt is made to guess a formal convention of adding
-    polar numbers, expressions like 1 + x will generally not be altered.
+    polar numbers, expressions like $1 + x$ will generally not be altered.
 
-    Note also that this function does not promote exp(x) to exp_polar(x).
+    Note also that this function does not promote ``exp(x)`` to ``exp_polar(x)``.
 
-    If ``subs`` is True, all symbols which are not already polar will be
+    If ``subs`` is ``True``, all symbols which are not already polar will be
     substituted for polar dummies; in this case the function behaves much
-    like posify.
+    like :func:`~.posify`.
 
-    If ``lift`` is True, both addition statements and non-polar symbols are
-    changed to their polar_lift()ed versions.
-    Note that lift=True implies subs=False.
+    If ``lift`` is ``True``, both addition statements and non-polar symbols are
+    changed to their ``polar_lift()``ed versions.
+    Note that ``lift=True`` implies ``subs=False``.
 
     Examples
     ========
@@ -1364,6 +1404,7 @@ def _unpolarify(eq, exponents_only, pause=False):
         return eq
 
     if not pause:
+        from sympy.functions.elementary.exponential import exp, exp_polar
         if isinstance(eq, exp_polar):
             return exp(_unpolarify(eq.exp, exponents_only))
         if isinstance(eq, principal_branch) and eq.args[1] == 2*pi:
@@ -1391,13 +1432,13 @@ def _unpolarify(eq, exponents_only, pause=False):
     return eq.func(*[_unpolarify(x, exponents_only, True) for x in eq.args])
 
 
-def unpolarify(eq, subs={}, exponents_only=False):
+def unpolarify(eq, subs=None, exponents_only=False):
     """
-    If p denotes the projection from the Riemann surface of the logarithm to
-    the complex line, return a simplified version eq' of `eq` such that
-    p(eq') == p(eq).
+    If `p` denotes the projection from the Riemann surface of the logarithm to
+    the complex line, return a simplified version `eq'` of `eq` such that
+    `p(eq') = p(eq)`.
     Also apply the substitution subs in the end. (This is a convenience, since
-    ``unpolarify``, in a certain sense, undoes polarify.)
+    ``unpolarify``, in a certain sense, undoes :func:`polarify`.)
 
     Examples
     ========
@@ -1412,7 +1453,7 @@ def unpolarify(eq, subs={}, exponents_only=False):
         return eq
 
     eq = sympify(eq)
-    if subs != {}:
+    if subs is not None:
         return unpolarify(eq.subs(subs))
     changed = True
     pause = False
@@ -1428,4 +1469,5 @@ def unpolarify(eq, subs={}, exponents_only=False):
             return res
     # Finally, replacing Exp(0) by 1 is always correct.
     # So is polar_lift(0) -> 0.
+    from sympy.functions.elementary.exponential import exp_polar
     return res.subs({exp_polar(0): 1, polar_lift(0): 0})
