@@ -1,9 +1,8 @@
-""" Elliptic integrals. """
-
-from __future__ import print_function, division
+""" Elliptic Integrals. """
 
 from sympy.core import S, pi, I, Rational
 from sympy.core.function import Function, ArgumentIndexError
+from sympy.core.symbol import Dummy,uniquely_named_symbol
 from sympy.functions.elementary.complexes import sign
 from sympy.functions.elementary.hyperbolic import atanh
 from sympy.functions.elementary.miscellaneous import sqrt
@@ -17,21 +16,24 @@ class elliptic_k(Function):
 
     .. math:: K(m) = F\left(\tfrac{\pi}{2}\middle| m\right)
 
-    where `F\left(z\middle| m\right)` is the Legendre incomplete
+    where $F\left(z\middle| m\right)$ is the Legendre incomplete
     elliptic integral of the first kind.
 
-    The function `K(m)` is a single-valued function on the complex
-    plane with branch cut along the interval `(1, \infty)`.
+    Explanation
+    ===========
+
+    The function $K(m)$ is a single-valued function on the complex
+    plane with branch cut along the interval $(1, \infty)$.
 
     Note that our notation defines the incomplete elliptic integral
-    in terms of the parameter `m` instead of the elliptic modulus
-    (eccentricity) `k`.
-    In this case, the parameter `m` is defined as `m=k^2`.
+    in terms of the parameter $m$ instead of the elliptic modulus
+    (eccentricity) $k$.
+    In this case, the parameter $m$ is defined as $m=k^2$.
 
     Examples
     ========
 
-    >>> from sympy import elliptic_k, I, pi
+    >>> from sympy import elliptic_k, I
     >>> from sympy.abc import m
     >>> elliptic_k(0)
     pi/2
@@ -49,14 +51,14 @@ class elliptic_k(Function):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Elliptic_integrals
-    .. [2] http://functions.wolfram.com/EllipticIntegrals/EllipticK
+    .. [2] https://functions.wolfram.com/EllipticIntegrals/EllipticK
 
     """
 
     @classmethod
     def eval(cls, m):
         if m.is_zero:
-            return pi/2
+            return pi*S.Half
         elif m is S.Half:
             return 8*pi**Rational(3, 2)/gamma(Rational(-1, 4))**2
         elif m is S.One:
@@ -67,9 +69,6 @@ class elliptic_k(Function):
                    I*S.NegativeInfinity, S.ComplexInfinity):
             return S.Zero
 
-        if m.is_zero:
-            return pi*S.Half
-
     def fdiff(self, argindex=1):
         m = self.args[0]
         return (elliptic_e(m) - (1 - m)*elliptic_k(m))/(2*m*(1 - m))
@@ -79,7 +78,7 @@ class elliptic_k(Function):
         if (m.is_real and (m - 1).is_positive) is False:
             return self.func(m.conjugate())
 
-    def _eval_nseries(self, x, n, logx):
+    def _eval_nseries(self, x, n, logx, cdir=0):
         from sympy.simplify import hyperexpand
         return hyperexpand(self.rewrite(hyper)._eval_nseries(x, n=n, logx=logx))
 
@@ -94,16 +93,12 @@ class elliptic_k(Function):
         if m.is_infinite:
             return True
 
-    def _eval_rewrite_as_Integral(self, *args):
-        from sympy import Integral, Dummy
-        t = Dummy('t')
+    def _eval_rewrite_as_Integral(self, *args, **kwargs):
+        from sympy.integrals.integrals import Integral
+        t = Dummy(uniquely_named_symbol('t', args).name)
         m = self.args[0]
         return Integral(1/sqrt(1 - m*sin(t)**2), (t, 0, pi/2))
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.elliptic_kc(self.args[0]._sage_())
-
 
 class elliptic_f(Function):
     r"""
@@ -113,18 +108,21 @@ class elliptic_f(Function):
     .. math:: F\left(z\middle| m\right) =
               \int_0^z \frac{dt}{\sqrt{1 - m \sin^2 t}}
 
+    Explanation
+    ===========
+
     This function reduces to a complete elliptic integral of
-    the first kind, `K(m)`, when `z = \pi/2`.
+    the first kind, $K(m)$, when $z = \pi/2$.
 
     Note that our notation defines the incomplete elliptic integral
-    in terms of the parameter `m` instead of the elliptic modulus
-    (eccentricity) `k`.
-    In this case, the parameter `m` is defined as `m=k^2`.
+    in terms of the parameter $m$ instead of the elliptic modulus
+    (eccentricity) $k$.
+    In this case, the parameter $m$ is defined as $m=k^2$.
 
     Examples
     ========
 
-    >>> from sympy import elliptic_f, I, O
+    >>> from sympy import elliptic_f, I
     >>> from sympy.abc import z, m
     >>> elliptic_f(z, m).series(z)
     z + z**5*(3*m**2/40 - m/30) + m*z**3/6 + O(z**6)
@@ -140,7 +138,7 @@ class elliptic_f(Function):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Elliptic_integrals
-    .. [2] http://functions.wolfram.com/EllipticIntegrals/EllipticF
+    .. [2] https://functions.wolfram.com/EllipticIntegrals/EllipticF
 
     """
 
@@ -173,9 +171,9 @@ class elliptic_f(Function):
         if (m.is_real and (m - 1).is_positive) is False:
             return self.func(z.conjugate(), m.conjugate())
 
-    def _eval_rewrite_as_Integral(self, *args):
-        from sympy import Integral, Dummy
-        t = Dummy('t')
+    def _eval_rewrite_as_Integral(self, *args, **kwargs):
+        from sympy.integrals.integrals import Integral
+        t = Dummy(uniquely_named_symbol('t', args).name)
         z, m = self.args[0], self.args[1]
         return Integral(1/(sqrt(1 - m*sin(t)**2)), (t, 0, z))
 
@@ -189,28 +187,31 @@ class elliptic_f(Function):
 
 class elliptic_e(Function):
     r"""
-    Called with two arguments `z` and `m`, evaluates the
+    Called with two arguments $z$ and $m$, evaluates the
     incomplete elliptic integral of the second kind, defined by
 
     .. math:: E\left(z\middle| m\right) = \int_0^z \sqrt{1 - m \sin^2 t} dt
 
-    Called with a single argument `m`, evaluates the Legendre complete
+    Called with a single argument $m$, evaluates the Legendre complete
     elliptic integral of the second kind
 
     .. math:: E(m) = E\left(\tfrac{\pi}{2}\middle| m\right)
 
-    The function `E(m)` is a single-valued function on the complex
-    plane with branch cut along the interval `(1, \infty)`.
+    Explanation
+    ===========
+
+    The function $E(m)$ is a single-valued function on the complex
+    plane with branch cut along the interval $(1, \infty)$.
 
     Note that our notation defines the incomplete elliptic integral
-    in terms of the parameter `m` instead of the elliptic modulus
-    (eccentricity) `k`.
-    In this case, the parameter `m` is defined as `m=k^2`.
+    in terms of the parameter $m$ instead of the elliptic modulus
+    (eccentricity) $k$.
+    In this case, the parameter $m$ is defined as $m=k^2$.
 
     Examples
     ========
 
-    >>> from sympy import elliptic_e, I, pi, O
+    >>> from sympy import elliptic_e, I
     >>> from sympy.abc import z, m
     >>> elliptic_e(z, m).series(z)
     z + z**5*(-m**2/40 + m/30) - m*z**3/6 + O(z**6)
@@ -227,8 +228,8 @@ class elliptic_e(Function):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Elliptic_integrals
-    .. [2] http://functions.wolfram.com/EllipticIntegrals/EllipticE2
-    .. [3] http://functions.wolfram.com/EllipticIntegrals/EllipticE
+    .. [2] https://functions.wolfram.com/EllipticIntegrals/EllipticE2
+    .. [3] https://functions.wolfram.com/EllipticIntegrals/EllipticE
 
     """
 
@@ -282,11 +283,11 @@ class elliptic_e(Function):
             if (m.is_real and (m - 1).is_positive) is False:
                 return self.func(m.conjugate())
 
-    def _eval_nseries(self, x, n, logx):
+    def _eval_nseries(self, x, n, logx, cdir=0):
         from sympy.simplify import hyperexpand
         if len(self.args) == 1:
             return hyperexpand(self.rewrite(hyper)._eval_nseries(x, n=n, logx=logx))
-        return super(elliptic_e, self)._eval_nseries(x, n=n, logx=logx)
+        return super()._eval_nseries(x, n=n, logx=logx)
 
     def _eval_rewrite_as_hyper(self, *args, **kwargs):
         if len(args) == 1:
@@ -299,36 +300,39 @@ class elliptic_e(Function):
             return -meijerg(((S.Half, Rational(3, 2)), []), \
                             ((S.Zero,), (S.Zero,)), -m)/4
 
-    def _eval_rewrite_as_Integral(self, *args):
-        from sympy import Integral, Dummy
+    def _eval_rewrite_as_Integral(self, *args, **kwargs):
+        from sympy.integrals.integrals import Integral
         z, m = (pi/2, self.args[0]) if len(self.args) == 1 else self.args
-        t = Dummy('t')
+        t = Dummy(uniquely_named_symbol('t', args).name)
         return Integral(sqrt(1 - m*sin(t)**2), (t, 0, z))
 
 
 class elliptic_pi(Function):
     r"""
-    Called with three arguments `n`, `z` and `m`, evaluates the
+    Called with three arguments $n$, $z$ and $m$, evaluates the
     Legendre incomplete elliptic integral of the third kind, defined by
 
     .. math:: \Pi\left(n; z\middle| m\right) = \int_0^z \frac{dt}
               {\left(1 - n \sin^2 t\right) \sqrt{1 - m \sin^2 t}}
 
-    Called with two arguments `n` and `m`, evaluates the complete
+    Called with two arguments $n$ and $m$, evaluates the complete
     elliptic integral of the third kind:
 
     .. math:: \Pi\left(n\middle| m\right) =
               \Pi\left(n; \tfrac{\pi}{2}\middle| m\right)
 
+    Explanation
+    ===========
+
     Note that our notation defines the incomplete elliptic integral
-    in terms of the parameter `m` instead of the elliptic modulus
-    (eccentricity) `k`.
-    In this case, the parameter `m` is defined as `m=k^2`.
+    in terms of the parameter $m$ instead of the elliptic modulus
+    (eccentricity) $k$.
+    In this case, the parameter $m$ is defined as $m=k^2$.
 
     Examples
     ========
 
-    >>> from sympy import elliptic_pi, I, pi, O, S
+    >>> from sympy import elliptic_pi, I
     >>> from sympy.abc import z, n, m
     >>> elliptic_pi(n, z, m).series(z, n=4)
     z + z**3*(m/6 + n/3) + O(z**4)
@@ -343,8 +347,8 @@ class elliptic_pi(Function):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Elliptic_integrals
-    .. [2] http://functions.wolfram.com/EllipticIntegrals/EllipticPi3
-    .. [3] http://functions.wolfram.com/EllipticIntegrals/EllipticPi
+    .. [2] https://functions.wolfram.com/EllipticIntegrals/EllipticPi3
+    .. [3] https://functions.wolfram.com/EllipticIntegrals/EllipticPi
 
     """
 
@@ -431,11 +435,11 @@ class elliptic_pi(Function):
                 return (elliptic_e(m)/(m - 1) + elliptic_pi(n, m))/(2*(n - m))
         raise ArgumentIndexError(self, argindex)
 
-    def _eval_rewrite_as_Integral(self, *args):
-        from sympy import Integral, Dummy
+    def _eval_rewrite_as_Integral(self, *args, **kwargs):
+        from sympy.integrals.integrals import Integral
         if len(self.args) == 2:
             n, m, z = self.args[0], self.args[1], pi/2
         else:
             n, z, m = self.args
-        t = Dummy('t')
+        t = Dummy(uniquely_named_symbol('t', args).name)
         return Integral(1/((1 - n*sin(t)**2)*sqrt(1 - m*sin(t)**2)), (t, 0, z))
