@@ -2,7 +2,7 @@ from sympy.core.function import expand_mul
 from sympy.core.symbol import Dummy, uniquely_named_symbol, symbols
 from sympy.utilities.iterables import numbered_symbols
 
-from .common import ShapeError, NonSquareMatrixError, NonInvertibleMatrixError
+from .exceptions import ShapeError, NonSquareMatrixError, NonInvertibleMatrixError
 from .eigen import _fuzzy_positive_definite
 from .utilities import _get_intermediate_simp, _iszero
 
@@ -14,7 +14,7 @@ def _diagonal_solve(M, rhs):
     Examples
     ========
 
-    >>> from sympy.matrices import Matrix, eye
+    >>> from sympy import Matrix, eye
     >>> A = eye(2)*2
     >>> B = Matrix([[1, 2], [3, 4]])
     >>> A.diagonal_solve(B) == B/2
@@ -31,12 +31,13 @@ def _diagonal_solve(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     if not M.is_diagonal():
         raise TypeError("Matrix should be diagonal")
     if rhs.rows != M.rows:
-        raise TypeError("Size mis-match")
+        raise TypeError("Size mismatch")
 
     return M._new(
         rhs.rows, rhs.cols, lambda i, j: rhs[i, j] / M[i, i])
@@ -56,6 +57,7 @@ def _lower_triangular_solve(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     from .dense import MutableDenseMatrix
@@ -94,6 +96,7 @@ def _lower_triangular_solve_sparse(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     if not M.is_square:
@@ -136,6 +139,7 @@ def _upper_triangular_solve(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     from .dense import MutableDenseMatrix
@@ -174,6 +178,7 @@ def _upper_triangular_solve_sparse(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     if not M.is_square:
@@ -219,6 +224,7 @@ def _cholesky_solve(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     if M.rows < M.cols:
@@ -258,7 +264,7 @@ def _LDLsolve(M, rhs):
     Examples
     ========
 
-    >>> from sympy.matrices import Matrix, eye
+    >>> from sympy import Matrix, eye
     >>> A = eye(2)*2
     >>> B = Matrix([[1, 2], [3, 4]])
     >>> A.LDLsolve(B) == B/2
@@ -276,6 +282,7 @@ def _LDLsolve(M, rhs):
     LUsolve
     QRsolve
     pinv_solve
+    cramer_solve
     """
 
     if M.rows < M.cols:
@@ -324,6 +331,7 @@ def _LUsolve(M, rhs, iszerofunc=_iszero):
     QRsolve
     pinv_solve
     LUdecomposition
+    cramer_solve
     """
 
     if rhs.rows != M.rows:
@@ -338,7 +346,7 @@ def _LUsolve(M, rhs, iszerofunc=_iszero):
 
     try:
         A, perm = M.LUdecomposition_Simple(
-            iszerofunc=_iszero, rankcheck=True)
+            iszerofunc=iszerofunc, rankcheck=True)
     except ValueError:
         raise NonInvertibleMatrixError("Matrix det == 0; not invertible.")
 
@@ -382,7 +390,7 @@ def _QRsolve(M, b):
 
     This method is slower (approximately by a factor of 2) but
     more stable for floating-point arithmetic than the LUsolve method.
-    However, LUsolve usually uses an exact arithmetic, so you don't need
+    However, LUsolve usually uses an exact arithmetic, so you do not need
     to use QRsolve.
 
     This is mainly for educational purposes and symbolic matrices, for real
@@ -400,6 +408,7 @@ def _QRsolve(M, b):
     LUsolve
     pinv_solve
     QRdecomposition
+    cramer_solve
     """
 
     dps  = _get_intermediate_simp(expand_mul, expand_mul)
@@ -581,7 +590,7 @@ def _gauss_jordan_solve(M, B, freevar=False):
 
     # Free parameters
     # what are current unnumbered free symbol names?
-    name = uniquely_named_symbol('tau', aug,
+    name = uniquely_named_symbol('tau', [aug],
             compare=lambda i: str(i).rstrip('1234567890'),
             modify=lambda s: '_' + s).name
     gen  = numbered_symbols(name)
@@ -699,6 +708,73 @@ def _pinv_solve(M, B, arbitrary_matrix=None):
             A_pinv.multiply(A)).multiply(arbitrary_matrix)
 
 
+def _cramer_solve(M, rhs, det_method="laplace"):
+    """Solves system of linear equations using Cramer's rule.
+
+    This method is relatively inefficient compared to other methods.
+    However it only uses a single division, assuming a division-free determinant
+    method is provided. This is helpful to minimize the chance of divide-by-zero
+    cases in symbolic solutions to linear systems.
+
+    Parameters
+    ==========
+    M : Matrix
+        The matrix representing the left hand side of the equation.
+    rhs : Matrix
+        The matrix representing the right hand side of the equation.
+    det_method : str or callable
+        The method to use to calculate the determinant of the matrix.
+        The default is ``'laplace'``.  If a callable is passed, it should take a
+        single argument, the matrix, and return the determinant of the matrix.
+
+    Returns
+    =======
+    x : Matrix
+        The matrix that will satisfy ``Ax = B``.  Will have as many rows as
+        matrix A has columns, and as many columns as matrix B.
+
+    Examples
+    ========
+
+    >>> from sympy import Matrix
+    >>> A = Matrix([[0, -6, 1], [0, -6, -1], [-5, -2, 3]])
+    >>> B = Matrix([[-30, -9], [-18, -27], [-26, 46]])
+    >>> x = A.cramer_solve(B)
+    >>> x
+    Matrix([
+    [ 0, -5],
+    [ 4,  3],
+    [-6,  9]])
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Cramer%27s_rule#Explicit_formulas_for_small_systems
+
+    """
+    from .dense import zeros
+
+    def entry(i, j):
+        return rhs[i, sol] if j == col else M[i, j]
+
+    if det_method == "bird":
+        from .determinant import _det_bird
+        det = _det_bird
+    elif det_method == "laplace":
+        from .determinant import _det_laplace
+        det = _det_laplace
+    elif isinstance(det_method, str):
+        det = lambda matrix: matrix.det(method=det_method)
+    else:
+        det = det_method
+    det_M = det(M)
+    x = zeros(*rhs.shape)
+    for sol in range(rhs.shape[1]):
+        for col in range(rhs.shape[0]):
+            x[col, sol] = det(M.__class__(*M.shape, entry)) / det_M
+    return M.__class__(x)
+
+
 def _solve(M, rhs, method='GJ'):
     """Solves linear equation where the unique solution exists.
 
@@ -718,6 +794,8 @@ def _solve(M, rhs, method='GJ'):
 
         If set to ``'PINV'``, ``pinv_solve`` routine will be used.
 
+        If set to ``'CRAMER'``, ``cramer_solve`` routine will be used.
+
         It also supports the methods available for special linear systems
 
         For positive definite systems:
@@ -746,7 +824,7 @@ def _solve(M, rhs, method='GJ'):
         for solving the system will be suggested.
     """
 
-    if method == 'GJ' or method == 'GE':
+    if method in ('GJ', 'GE'):
         try:
             soln, param = M.gauss_jordan_solve(rhs)
 
@@ -769,6 +847,8 @@ def _solve(M, rhs, method='GJ'):
         return M.LDLsolve(rhs)
     elif method == 'PINV':
         return M.pinv_solve(rhs)
+    elif method == 'CRAMER':
+        return M.cramer_solve(rhs)
     else:
         return M.inv(method=method).multiply(rhs)
 
@@ -804,7 +884,7 @@ def _solve_least_squares(M, rhs, method='CH'):
     Examples
     ========
 
-    >>> from sympy.matrices import Matrix, ones
+    >>> from sympy import Matrix, ones
     >>> A = Matrix([1, 2, 3])
     >>> B = Matrix([2, 3, 4])
     >>> S = Matrix(A.row_join(B))
