@@ -4,9 +4,9 @@ from sympy.polys.partfrac import apart
 from sympy.core.symbol import Dummy
 from sympy.external import import_module
 from sympy.functions import arg, Abs
-from sympy.integrals.transforms import _fast_inverse_laplace
+from sympy.integrals.laplace import _fast_inverse_laplace
 from sympy.physics.control.lti import SISOLinearTimeInvariant
-from sympy.plotting.plot import LineOver1DRangeSeries
+from sympy.plotting.series import LineOver1DRangeSeries
 from sympy.polys.polytools import Poly
 from sympy.printing.latex import latex
 
@@ -210,8 +210,8 @@ def step_response_numerical_data(system, prec=8, lower_limit=0,
     of a SISO continuous-time system. By default, adaptive sampling
     is used. If the user wants to instead get an uniformly
     sampled response, then ``adaptive`` kwarg should be passed ``False``
-    and ``nb_of_points`` must be passed as additional kwargs.
-    Refer to the parameters of class :class:`sympy.plotting.plot.LineOver1DRangeSeries`
+    and ``n`` must be passed as additional kwargs.
+    Refer to the parameters of class :class:`sympy.plotting.series.LineOver1DRangeSeries`
     for more details.
 
     Parameters
@@ -228,7 +228,7 @@ def step_response_numerical_data(system, prec=8, lower_limit=0,
         The upper limit of the plot range. Defaults to 10.
     kwargs :
         Additional keyword arguments are passed to the underlying
-        :class:`sympy.plotting.plot.LineOver1DRangeSeries` class.
+        :class:`sympy.plotting.series.LineOver1DRangeSeries` class.
 
     Returns
     =======
@@ -361,8 +361,8 @@ def impulse_response_numerical_data(system, prec=8, lower_limit=0,
     of a SISO continuous-time system. By default, adaptive sampling
     is used. If the user wants to instead get an uniformly
     sampled response, then ``adaptive`` kwarg should be passed ``False``
-    and ``nb_of_points`` must be passed as additional kwargs.
-    Refer to the parameters of class :class:`sympy.plotting.plot.LineOver1DRangeSeries`
+    and ``n`` must be passed as additional kwargs.
+    Refer to the parameters of class :class:`sympy.plotting.series.LineOver1DRangeSeries`
     for more details.
 
     Parameters
@@ -379,7 +379,7 @@ def impulse_response_numerical_data(system, prec=8, lower_limit=0,
         The upper limit of the plot range. Defaults to 10.
     kwargs :
         Additional keyword arguments are passed to the underlying
-        :class:`sympy.plotting.plot.LineOver1DRangeSeries` class.
+        :class:`sympy.plotting.series.LineOver1DRangeSeries` class.
 
     Returns
     =======
@@ -483,7 +483,7 @@ def impulse_response_plot(system, color='b', prec=8, lower_limit=0,
     References
     ==========
 
-    .. [1] https://www.mathworks.com/help/control/ref/lti.impulse.html
+    .. [1] https://www.mathworks.com/help/control/ref/dynamicsystem.impulse.html
 
     """
     x, y = impulse_response_numerical_data(system, prec=prec,
@@ -512,8 +512,8 @@ def ramp_response_numerical_data(system, slope=1, prec=8,
     of a SISO continuous-time system. By default, adaptive sampling
     is used. If the user wants to instead get an uniformly
     sampled response, then ``adaptive`` kwarg should be passed ``False``
-    and ``nb_of_points`` must be passed as additional kwargs.
-    Refer to the parameters of class :class:`sympy.plotting.plot.LineOver1DRangeSeries`
+    and ``n`` must be passed as additional kwargs.
+    Refer to the parameters of class :class:`sympy.plotting.series.LineOver1DRangeSeries`
     for more details.
 
     Parameters
@@ -532,7 +532,7 @@ def ramp_response_numerical_data(system, slope=1, prec=8,
         The upper limit of the plot range. Defaults to 10.
     kwargs :
         Additional keyword arguments are passed to the underlying
-        :class:`sympy.plotting.plot.LineOver1DRangeSeries` class.
+        :class:`sympy.plotting.series.LineOver1DRangeSeries` class.
 
     Returns
     =======
@@ -642,7 +642,7 @@ def ramp_response_plot(system, slope=1, color='b', prec=8, lower_limit=0,
     See Also
     ========
 
-    step_response_plot, ramp_response_plot
+    step_response_plot, impulse_response_plot
 
     References
     ==========
@@ -778,7 +778,7 @@ def bode_magnitude_plot(system, initial_exp=-5, final_exp=5,
     return plt
 
 
-def bode_phase_numerical_data(system, initial_exp=-5, final_exp=5, freq_unit='rad/sec', phase_unit='rad', **kwargs):
+def bode_phase_numerical_data(system, initial_exp=-5, final_exp=5, freq_unit='rad/sec', phase_unit='rad', phase_unwrap = True, **kwargs):
     """
     Returns the numerical data of the Bode phase plot of the system.
     It is internally used by ``bode_phase_plot`` to get the data
@@ -799,6 +799,8 @@ def bode_phase_numerical_data(system, initial_exp=-5, final_exp=5, freq_unit='ra
         User can choose between ``'rad/sec'`` (radians/second) and '``'Hz'`` (Hertz) as frequency units.
     phase_unit : string, optional
         User can choose between ``'rad'`` (radians) and ``'deg'`` (degree) as phase units.
+    phase_unwrap : bool, optional
+        Set to ``True`` by default.
 
     Returns
     =======
@@ -863,18 +865,33 @@ def bode_phase_numerical_data(system, initial_exp=-5, final_exp=5, freq_unit='ra
     x, y = LineOver1DRangeSeries(phase,
         (_w, 10**initial_exp, 10**final_exp), xscale='log', **kwargs).get_points()
 
+    half = None
+    if phase_unwrap:
+        if(phase_unit == 'rad'):
+            half = pi
+        elif(phase_unit == 'deg'):
+            half = 180
+    if half:
+        unit = 2*half
+        for i in range(1, len(y)):
+            diff = y[i] - y[i - 1]
+            if diff > half:      # Jump from -half to half
+                y[i] = (y[i] - unit)
+            elif diff < -half:   # Jump from half to -half
+                y[i] = (y[i] + unit)
+
     return x, y
 
 
 def bode_phase_plot(system, initial_exp=-5, final_exp=5,
-    color='b', show_axes=False, grid=True, show=True, freq_unit='rad/sec', phase_unit='rad', **kwargs):
+    color='b', show_axes=False, grid=True, show=True, freq_unit='rad/sec', phase_unit='rad', phase_unwrap=True, **kwargs):
     r"""
     Returns the Bode phase plot of a continuous-time system.
 
     See ``bode_plot`` for all the parameters.
     """
     x, y = bode_phase_numerical_data(system, initial_exp=initial_exp,
-        final_exp=final_exp, freq_unit=freq_unit, phase_unit=phase_unit)
+        final_exp=final_exp, freq_unit=freq_unit, phase_unit=phase_unit, phase_unwrap=phase_unwrap)
     plt.plot(x, y, color=color, **kwargs)
     plt.xscale('log')
 
@@ -895,7 +912,7 @@ def bode_phase_plot(system, initial_exp=-5, final_exp=5,
 
 
 def bode_plot(system, initial_exp=-5, final_exp=5,
-    grid=True, show_axes=False, show=True, freq_unit='rad/sec', phase_unit='rad', **kwargs):
+    grid=True, show_axes=False, show=True, freq_unit='rad/sec', phase_unit='rad', phase_unwrap=True, **kwargs):
     r"""
     Returns the Bode phase and magnitude plots of a continuous-time system.
 
@@ -952,7 +969,7 @@ def bode_plot(system, initial_exp=-5, final_exp=5,
     mag.xlabel(None)
     plt.subplot(212)
     bode_phase_plot(system, initial_exp=initial_exp, final_exp=final_exp,
-        show=False, grid=grid, show_axes=show_axes, freq_unit=freq_unit, phase_unit=phase_unit, **kwargs).title(None)
+        show=False, grid=grid, show_axes=show_axes, freq_unit=freq_unit, phase_unit=phase_unit, phase_unwrap=phase_unwrap, **kwargs).title(None)
 
     if show:
         plt.show()
