@@ -1,6 +1,9 @@
 """Utilities to deal with sympy.Matrix, numpy and scipy.sparse."""
 
-from sympy import MatrixBase, I, Expr, Integer
+from sympy.core.expr import Expr
+from sympy.core.numbers import I
+from sympy.core.singleton import S
+from sympy.matrices.matrixbase import MatrixBase
 from sympy.matrices import eye, zeros
 from sympy.external import import_module
 
@@ -37,22 +40,16 @@ if not scipy:
     sparse = None
 else:
     sparse = scipy.sparse
-    # Try to find spmatrix.
-    if hasattr(sparse, 'base'):
-        # Newer versions have it under scipy.sparse.base.
-        scipy_sparse_matrix = sparse.base.spmatrix  # type: ignore
-    elif hasattr(sparse, 'sparse'):
-        # Older versions have it under scipy.sparse.sparse.
-        scipy_sparse_matrix = sparse.sparse.spmatrix  # type: ignore
+    scipy_sparse_matrix = sparse.spmatrix # type: ignore
 
 
 def sympy_to_numpy(m, **options):
-    """Convert a sympy Matrix/complex number to a numpy matrix or scalar."""
+    """Convert a SymPy Matrix/complex number to a numpy matrix or scalar."""
     if not np:
         raise ImportError
     dtype = options.get('dtype', 'complex')
     if isinstance(m, MatrixBase):
-        return np.matrix(m.tolist(), dtype=dtype)
+        return np.array(m.tolist(), dtype=dtype)
     elif isinstance(m, Expr):
         if m.is_Number or m.is_NumberSymbol or m == I:
             return complex(m)
@@ -60,12 +57,12 @@ def sympy_to_numpy(m, **options):
 
 
 def sympy_to_scipy_sparse(m, **options):
-    """Convert a sympy Matrix/complex number to a numpy matrix or scalar."""
+    """Convert a SymPy Matrix/complex number to a numpy matrix or scalar."""
     if not np or not sparse:
         raise ImportError
     dtype = options.get('dtype', 'complex')
     if isinstance(m, MatrixBase):
-        return sparse.csr_matrix(np.matrix(m.tolist(), dtype=dtype))
+        return sparse.csr_matrix(np.array(m.tolist(), dtype=dtype))
     elif isinstance(m, Expr):
         if m.is_Number or m.is_NumberSymbol or m == I:
             return complex(m)
@@ -73,17 +70,17 @@ def sympy_to_scipy_sparse(m, **options):
 
 
 def scipy_sparse_to_sympy(m, **options):
-    """Convert a scipy.sparse matrix to a sympy matrix."""
+    """Convert a scipy.sparse matrix to a SymPy matrix."""
     return MatrixBase(m.todense())
 
 
 def numpy_to_sympy(m, **options):
-    """Convert a numpy matrix to a sympy matrix."""
+    """Convert a numpy matrix to a SymPy matrix."""
     return MatrixBase(m)
 
 
 def to_sympy(m, **options):
-    """Convert a numpy/scipy.sparse matrix to a sympy matrix."""
+    """Convert a numpy/scipy.sparse matrix to a SymPy matrix."""
     if isinstance(m, MatrixBase):
         return m
     elif isinstance(m, numpy_ndarray):
@@ -143,7 +140,7 @@ def matrix_dagger(e):
 
 # TODO: Move this into sympy.matricies.
 def _sympy_tensor_product(*matrices):
-    """Compute the kronecker product of a sequence of sympy Matrices.
+    """Compute the kronecker product of a sequence of SymPy Matrices.
     """
     from sympy.matrices.expressions.kronecker import matrix_kronecker_product
 
@@ -186,7 +183,7 @@ def _numpy_eye(n):
     """numpy version of complex eye."""
     if not np:
         raise ImportError
-    return np.matrix(np.eye(n, dtype='complex'))
+    return np.array(np.eye(n, dtype='complex'))
 
 
 def _scipy_sparse_eye(n):
@@ -267,7 +264,7 @@ def matrix_to_zero(e):
     """Convert a zero matrix to the scalar zero."""
     if isinstance(e, MatrixBase):
         if zeros(*e.shape) == e:
-            e = Integer(0)
+            e = S.Zero
     elif isinstance(e, numpy_ndarray):
         e = _numpy_matrix_to_zero(e)
     elif isinstance(e, scipy_sparse_matrix):
