@@ -6,7 +6,12 @@ TODO:
 * Document default basis functionality.
 """
 
-from sympy import Add, Expr, I, integrate, Mul, Pow
+from sympy.core.add import Add
+from sympy.core.expr import Expr
+from sympy.core.mul import Mul
+from sympy.core.numbers import I
+from sympy.core.power import Pow
+from sympy.integrals.integrals import integrate
 from sympy.physics.quantum.dagger import Dagger
 from sympy.physics.quantum.commutator import Commutator
 from sympy.physics.quantum.anticommutator import AntiCommutator
@@ -34,7 +39,7 @@ __all__ = [
 
 
 def _sympy_to_scalar(e):
-    """Convert from a sympy scalar to a Python scalar."""
+    """Convert from a SymPy scalar to a Python scalar."""
     if isinstance(e, Expr):
         if e.is_Integer:
             return int(e)
@@ -59,7 +64,7 @@ def represent(expr, **options):
 
     This function is the top-level interface for this action.
 
-    This function walks the sympy expression tree looking for ``QExpr``
+    This function walks the SymPy expression tree looking for ``QExpr``
     instances that have a ``_represent`` method. This method is then called
     and the object is replaced by the representation returned by this method.
     By default, the ``_represent`` method will dispatch to other methods
@@ -134,6 +139,8 @@ def represent(expr, **options):
     """
 
     format = options.get('format', 'sympy')
+    if format == 'numpy':
+        import numpy as np
     if isinstance(expr, QExpr) and not isinstance(expr, OuterProduct):
         options['replace_none'] = False
         temp_basis = get_basis(expr, **options)
@@ -167,7 +174,7 @@ def represent(expr, **options):
         return result
     elif isinstance(expr, Pow):
         base, exp = expr.as_base_exp()
-        if format == 'numpy' or format == 'scipy.sparse':
+        if format in ('numpy', 'scipy.sparse'):
             exp = _sympy_to_scalar(exp)
         base = represent(base, **options)
         # scipy.sparse doesn't support negative exponents
@@ -176,6 +183,8 @@ def represent(expr, **options):
             from scipy.sparse.linalg import inv
             exp = - exp
             base = inv(base.tocsc()).tocsr()
+        if format == 'numpy':
+            return np.linalg.matrix_power(base, exp)
         return base ** exp
     elif isinstance(expr, TensorProduct):
         new_args = [represent(arg, **options) for arg in expr.args]
@@ -183,22 +192,22 @@ def represent(expr, **options):
     elif isinstance(expr, Dagger):
         return Dagger(represent(expr.args[0], **options))
     elif isinstance(expr, Commutator):
-        A = represent(expr.args[0], **options)
-        B = represent(expr.args[1], **options)
-        return A*B - B*A
+        A = expr.args[0]
+        B = expr.args[1]
+        return represent(Mul(A, B) - Mul(B, A), **options)
     elif isinstance(expr, AntiCommutator):
-        A = represent(expr.args[0], **options)
-        B = represent(expr.args[1], **options)
-        return A*B + B*A
+        A = expr.args[0]
+        B = expr.args[1]
+        return represent(Mul(A, B) + Mul(B, A), **options)
     elif isinstance(expr, InnerProduct):
         return represent(Mul(expr.bra, expr.ket), **options)
-    elif not (isinstance(expr, Mul) or isinstance(expr, OuterProduct)):
+    elif not isinstance(expr, (Mul, OuterProduct)):
         # For numpy and scipy.sparse, we can only handle numerical prefactors.
-        if format == 'numpy' or format == 'scipy.sparse':
+        if format in ('numpy', 'scipy.sparse'):
             return _sympy_to_scalar(expr)
         return expr
 
-    if not (isinstance(expr, Mul) or isinstance(expr, OuterProduct)):
+    if not isinstance(expr, (Mul, OuterProduct)):
         raise TypeError('Mul expected, got: %r' % expr)
 
     if "index" in options:
@@ -206,7 +215,7 @@ def represent(expr, **options):
     else:
         options["index"] = 1
 
-    if not "unities" in options:
+    if "unities" not in options:
         options["unities"] = []
 
     result = represent(expr.args[-1], **options)
@@ -223,7 +232,12 @@ def represent(expr, **options):
         elif isinstance(last_arg, KetBase) and isinstance(arg, BraBase):
             options["unities"].append(options["index"])
 
-        result = represent(arg, **options)*result
+        next_arg = represent(arg, **options)
+        if format == 'numpy' and isinstance(next_arg, np.ndarray):
+            # Must use np.matmult to "matrix multiply" two np.ndarray
+            result = np.matmul(next_arg, result)
+        else:
+            result = next_arg*result
         last_arg = arg
 
     # All three matrix formats create 1 by 1 matrices when inner products of
@@ -271,7 +285,7 @@ def rep_innerproduct(expr, **options):
     if not isinstance(basis, StateBase):
         raise NotImplementedError("Can't form this representation!")
 
-    if not "index" in options:
+    if "index" not in options:
         options["index"] = 1
 
     basis_kets = enumerate_states(basis, options["index"], 2)
@@ -315,7 +329,7 @@ def rep_expectation(expr, **options):
 
     """
 
-    if not "index" in options:
+    if "index" not in options:
         options["index"] = 1
 
     if not isinstance(expr, Operator):
@@ -379,7 +393,7 @@ def integrate_result(orig_expr, result, **options):
         return result
 
     options['replace_none'] = True
-    if not "basis" in options:
+    if "basis" not in options:
         arg = orig_expr.args[-1]
         options["basis"] = get_basis(arg, **options)
     elif not isinstance(options["basis"], StateBase):
@@ -539,7 +553,7 @@ def enumerate_states(*args, **options):
 
     state = args[0]
 
-    if not (len(args) == 2 or len(args) == 3):
+    if len(args) not in (2, 3):
         raise NotImplementedError("Wrong number of arguments!")
 
     if not isinstance(state, StateBase):
