@@ -1,15 +1,85 @@
 """Implementation of :class:`FiniteField` class. """
 
+import operator
 
+from sympy.external.gmpy import GROUND_TYPES
+from sympy.utilities.decorator import doctest_depends_on
+
+from sympy.core.numbers import int_valued
 from sympy.polys.domains.field import Field
 
 from sympy.polys.domains.modularinteger import ModularIntegerFactory
 from sympy.polys.domains.simpledomain import SimpleDomain
+from sympy.polys.galoistools import gf_zassenhaus, gf_irred_p_rabin
 from sympy.polys.polyerrors import CoercionFailed
 from sympy.utilities import public
 from sympy.polys.domains.groundtypes import SymPyInteger
 
+
+if GROUND_TYPES == 'flint':
+    __doctest_skip__ = ['FiniteField']
+
+
+if GROUND_TYPES == 'flint':
+    import flint
+    # Don't use python-flint < 0.5.0 because nmod was missing some features in
+    # previous versions of python-flint and fmpz_mod was not yet added.
+    _major, _minor, *_ = flint.__version__.split('.')
+    if (int(_major), int(_minor)) < (0, 5):
+        flint = None
+else:
+    flint = None
+
+
+def _modular_int_factory(mod, dom, symmetric, self):
+
+    # Use flint if available
+    if flint is not None:
+
+        nmod = flint.nmod
+        fmpz_mod_ctx = flint.fmpz_mod_ctx
+        index = operator.index
+
+        try:
+            mod = dom.convert(mod)
+        except CoercionFailed:
+            raise ValueError('modulus must be an integer, got %s' % mod)
+
+        # mod might be e.g. Integer
+        try:
+            fmpz_mod_ctx(mod)
+        except TypeError:
+            mod = index(mod)
+
+        # flint's nmod is only for moduli up to 2^64-1 (on a 64-bit machine)
+        try:
+            nmod(0, mod)
+        except OverflowError:
+            # Use fmpz_mod
+            fctx = fmpz_mod_ctx(mod)
+
+            def ctx(x):
+                try:
+                    return fctx(x)
+                except TypeError:
+                    # x might be Integer
+                    return fctx(index(x))
+        else:
+            # Use nmod
+            def ctx(x):
+                try:
+                    return nmod(x, mod)
+                except TypeError:
+                    return nmod(index(x), mod)
+
+        return ctx
+
+    # Use the Python implementation
+    return ModularIntegerFactory(mod, dom, symmetric, self)
+
+
 @public
+@doctest_depends_on(modules=['python', 'gmpy'])
 class FiniteField(Field, SimpleDomain):
     r"""Finite field of prime order :ref:`GF(p)`
 
@@ -18,8 +88,8 @@ class FiniteField(Field, SimpleDomain):
     :ref:`polys-domainsintro`).
 
     A :py:class:`~.Poly` created from an expression with integer
-    coefficients will have the domain :ref:`ZZ`. Howeer if the ``modulus=p``
-    option is given the the domain will be a finite field instead.
+    coefficients will have the domain :ref:`ZZ`. However, if the ``modulus=p``
+    option is given then the domain will be a finite field instead.
 
     >>> from sympy import Poly, Symbol
     >>> x = Symbol('x')
@@ -118,11 +188,17 @@ class FiniteField(Field, SimpleDomain):
         if mod <= 0:
             raise ValueError('modulus must be a positive integer, got %s' % mod)
 
-        self.dtype = ModularIntegerFactory(mod, dom, symmetric, self)
+        self.dtype = _modular_int_factory(mod, dom, symmetric, self)
         self.zero = self.dtype(0)
         self.one = self.dtype(1)
         self.dom = dom
         self.mod = mod
+        self.sym = symmetric
+        self._tp = type(self.zero)
+
+    @property
+    def tp(self):
+        return self._tp
 
     def __str__(self):
         return 'GF(%s)' % self.mod
@@ -145,24 +221,47 @@ class FiniteField(Field, SimpleDomain):
 
     def to_sympy(self, a):
         """Convert ``a`` to a SymPy object. """
-        return SymPyInteger(int(a))
+        return SymPyInteger(self.to_int(a))
 
     def from_sympy(self, a):
         """Convert SymPy's Integer to SymPy's ``Integer``. """
         if a.is_Integer:
             return self.dtype(self.dom.dtype(int(a)))
-        elif a.is_Float and int(a) == a:
+        elif int_valued(a):
             return self.dtype(self.dom.dtype(int(a)))
         else:
             raise CoercionFailed("expected an integer, got %s" % a)
 
+    def to_int(self, a):
+        """Convert ``val`` to a Python ``int`` object. """
+        aval = int(a)
+        if self.sym and aval > self.mod // 2:
+            aval -= self.mod
+        return aval
+
+    def is_positive(self, a):
+        """Returns True if ``a`` is positive. """
+        return bool(a)
+
+    def is_nonnegative(self, a):
+        """Returns True if ``a`` is non-negative. """
+        return True
+
+    def is_negative(self, a):
+        """Returns True if ``a`` is negative. """
+        return False
+
+    def is_nonpositive(self, a):
+        """Returns True if ``a`` is non-positive. """
+        return not a
+
     def from_FF(K1, a, K0=None):
         """Convert ``ModularInteger(int)`` to ``dtype``. """
-        return K1.dtype(K1.dom.from_ZZ(a.val, K0.dom))
+        return K1.dtype(K1.dom.from_ZZ(int(a), K0.dom))
 
     def from_FF_python(K1, a, K0=None):
         """Convert ``ModularInteger(int)`` to ``dtype``. """
-        return K1.dtype(K1.dom.from_ZZ_python(a.val, K0.dom))
+        return K1.dtype(K1.dom.from_ZZ_python(int(a), K0.dom))
 
     def from_ZZ(K1, a, K0=None):
         """Convert Python's ``int`` to ``dtype``. """
@@ -202,5 +301,28 @@ class FiniteField(Field, SimpleDomain):
         if q == 1:
             return K1.dtype(K1.dom.dtype(p))
 
+    def is_square(self, a):
+        """Returns True if ``a`` is a quadratic residue modulo p. """
+        # a is not a square <=> x**2-a is irreducible
+        poly = [int(x) for x in [self.one, self.zero, -a]]
+        return not gf_irred_p_rabin(poly, self.mod, self.dom)
+
+    def exsqrt(self, a):
+        """Square root modulo p of ``a`` if it is a quadratic residue.
+
+        Explanation
+        ===========
+        Always returns the square root that is no larger than ``p // 2``.
+        """
+        # x**2-a is not square-free if a=0 or the field is characteristic 2
+        if self.mod == 2 or a == 0:
+            return a
+        # Otherwise, use square-free factorization routine to factorize x**2-a
+        poly = [int(x) for x in [self.one, self.zero, -a]]
+        for factor in gf_zassenhaus(poly, self.mod, self.dom):
+            if len(factor) == 2 and factor[1] <= self.mod // 2:
+                return self.dtype(factor[1])
+        return None
+
 
 FF = GF = FiniteField
