@@ -1,6 +1,7 @@
 """Tests for Dixon's and Macaulay's classes. """
 
-from sympy import Matrix, factor
+from sympy.matrices.dense import Matrix
+from sympy.polys.polytools import factor
 from sympy.core import symbols
 from sympy.tensor.indexed import IndexedBase
 
