@@ -1,20 +1,22 @@
 """
 Javascript code printer
 
-The JavascriptCodePrinter converts single sympy expressions into single
+The JavascriptCodePrinter converts single SymPy expressions into single
 Javascript expressions, using the functions defined in the Javascript
 Math object where possible.
 
 """
 
-from typing import Any, Dict
+from __future__ import annotations
+from typing import Any
 
 from sympy.core import S
+from sympy.core.numbers import equal_valued
 from sympy.printing.codeprinter import CodePrinter
 from sympy.printing.precedence import precedence, PRECEDENCE
 
 
-# dictionary mapping sympy function to (argument_conditions, Javascript_function).
+# dictionary mapping SymPy function to (argument_conditions, Javascript_function).
 # Used in JavascriptCodePrinter._print_Function(self)
 known_functions = {
     'Abs': 'Math.abs',
@@ -42,20 +44,16 @@ known_functions = {
 
 
 class JavascriptCodePrinter(CodePrinter):
-    """"A Printer to convert python expressions to strings of javascript code
+    """"A Printer to convert Python expressions to strings of JavaScript code
     """
     printmethod = '_javascript'
-    language = 'Javascript'
+    language = 'JavaScript'
 
-    _default_settings = {
-        'order': None,
-        'full_prec': 'auto',
+    _default_settings: dict[str, Any] = dict(CodePrinter._default_settings, **{
         'precision': 17,
         'user_functions': {},
-        'human': True,
-        'allow_unknown_functions': False,
         'contract': True,
-    }  # type: Dict[str, Any]
+    })
 
     def __init__(self, settings={}):
         CodePrinter.__init__(self, settings)
@@ -97,9 +95,9 @@ class JavascriptCodePrinter(CodePrinter):
 
     def _print_Pow(self, expr):
         PREC = precedence(expr)
-        if expr.exp == -1:
+        if equal_valued(expr.exp, -1):
             return '1/%s' % (self.parenthesize(expr.base, PREC))
-        elif expr.exp == 0.5:
+        elif equal_valued(expr.exp, 0.5):
             return 'Math.sqrt(%s)' % self._print(expr.base)
         elif expr.exp == S.One/3:
             return 'Math.cbrt(%s)' % self._print(expr.base)
@@ -111,6 +109,18 @@ class JavascriptCodePrinter(CodePrinter):
         p, q = int(expr.p), int(expr.q)
         return '%d/%d' % (p, q)
 
+    def _print_Mod(self, expr):
+        num, den = expr.args
+        PREC = precedence(expr)
+        snum, sden = [self.parenthesize(arg, PREC) for arg in expr.args]
+        # % is remainder (same sign as numerator), not modulo (same sign as
+        # denominator), in js. Hence, % only works as modulo if both numbers
+        # have the same sign
+        if (num.is_nonnegative and den.is_nonnegative or
+            num.is_nonpositive and den.is_nonpositive):
+            return f"{snum} % {sden}"
+        return f"(({snum} % {sden}) + {sden}) % {sden}"
+
     def _print_Relational(self, expr):
         lhs_code = self._print(expr.lhs)
         rhs_code = self._print(expr.rhs)
@@ -200,7 +210,7 @@ class JavascriptCodePrinter(CodePrinter):
         pretty = []
         level = 0
         for n, line in enumerate(code):
-            if line == '' or line == '\n':
+            if line in ('', '\n'):
                 pretty.append(line)
                 continue
             level -= decrease[n]
@@ -216,7 +226,7 @@ def jscode(expr, assign_to=None, **settings):
     ==========
 
     expr : Expr
-        A sympy expression to be converted.
+        A SymPy expression to be converted.
     assign_to : optional
         When given, the argument is used as the name of the variable to which
         the expression is assigned. Can be a string, ``Symbol``,
