@@ -31,10 +31,11 @@ complete source code files.
 # .. _Rational64: http://rust-num.github.io/num/num/rational/type.Rational64.html
 # .. _BigRational: http://rust-num.github.io/num/num/rational/type.BigRational.html
 
-from __future__ import print_function, division
+from __future__ import annotations
+from typing import Any
 
 from sympy.core import S, Rational, Float, Lambda
-from sympy.core.compatibility import string_types, range
+from sympy.core.numbers import equal_valued
 from sympy.printing.codeprinter import CodePrinter
 
 # Rust's methods for integer and float can be found at here :
@@ -49,43 +50,43 @@ from sympy.printing.codeprinter import CodePrinter
 # 3. args[1].func(), method without arguments (e.g. (e, x) => x.exp())
 # 4. func(args), function with arguments
 
-# dictionary mapping sympy function to (argument_conditions, Rust_function).
+# dictionary mapping SymPy function to (argument_conditions, Rust_function).
 # Used in RustCodePrinter._print_Function(self)
 
 # f64 method in Rust
 known_functions = {
-    "": "is_nan",
-    "": "is_infinite",
-    "": "is_finite",
-    "": "is_normal",
-    "": "classify",
+    # "": "is_nan",
+    # "": "is_infinite",
+    # "": "is_finite",
+    # "": "is_normal",
+    # "": "classify",
     "floor": "floor",
     "ceiling": "ceil",
-    "": "round",
-    "": "trunc",
-    "": "fract",
+    # "": "round",
+    # "": "trunc",
+    # "": "fract",
     "Abs": "abs",
     "sign": "signum",
-    "": "is_sign_positive",
-    "": "is_sign_negative",
-    "": "mul_add",
-    "Pow": [(lambda base, exp: exp == -S.One, "recip", 2),           # 1.0/x
-            (lambda base, exp: exp == S.Half, "sqrt", 2),            # x ** 0.5
-            (lambda base, exp: exp == -S.Half, "sqrt().recip", 2),   # 1/(x ** 0.5)
+    # "": "is_sign_positive",
+    # "": "is_sign_negative",
+    # "": "mul_add",
+    "Pow": [(lambda base, exp: equal_valued(exp, -1), "recip", 2),   # 1.0/x
+            (lambda base, exp: equal_valued(exp, 0.5), "sqrt", 2),   # x ** 0.5
+            (lambda base, exp: equal_valued(exp, -0.5), "sqrt().recip", 2),   # 1/(x ** 0.5)
             (lambda base, exp: exp == Rational(1, 3), "cbrt", 2),    # x ** (1/3)
-            (lambda base, exp: base == S.One*2, "exp2", 3),          # 2 ** x
+            (lambda base, exp: equal_valued(base, 2), "exp2", 3),    # 2 ** x
             (lambda base, exp: exp.is_integer, "powi", 1),           # x ** y, for i32
             (lambda base, exp: not exp.is_integer, "powf", 1)],      # x ** y, for f64
     "exp": [(lambda exp: True, "exp", 2)],   # e ** x
     "log": "ln",
-    "": "log",          # number.log(base)
-    "": "log2",
-    "": "log10",
-    "": "to_degrees",
-    "": "to_radians",
+    # "": "log",          # number.log(base)
+    # "": "log2",
+    # "": "log10",
+    # "": "to_degrees",
+    # "": "to_radians",
     "Max": "max",
     "Min": "min",
-    "": "hypot",        # (x**2 + y**2) ** 0.5
+    # "": "hypot",        # (x**2 + y**2) ** 0.5
     "sin": "sin",
     "cos": "cos",
     "tan": "tan",
@@ -93,15 +94,16 @@ known_functions = {
     "acos": "acos",
     "atan": "atan",
     "atan2": "atan2",
-    "": "sin_cos",
-    "": "exp_m1",       # e ** x - 1
-    "": "ln_1p",        # ln(1 + x)
+    # "": "sin_cos",
+    # "": "exp_m1",       # e ** x - 1
+    # "": "ln_1p",        # ln(1 + x)
     "sinh": "sinh",
     "cosh": "cosh",
     "tanh": "tanh",
     "asinh": "asinh",
     "acosh": "acosh",
     "atanh": "atanh",
+    "sqrt": "sqrt",  # To enable automatic rewrites
 }
 
 # i64 method in Rust
@@ -215,22 +217,16 @@ reserved_words = ['abstract',
 
 
 class RustCodePrinter(CodePrinter):
-    """A printer to convert python expressions to strings of Rust code"""
+    """A printer to convert SymPy expressions to strings of Rust code"""
     printmethod = "_rust_code"
     language = "Rust"
 
-    _default_settings = {
-        'order': None,
-        'full_prec': 'auto',
+    _default_settings: dict[str, Any] = dict(CodePrinter._default_settings, **{
         'precision': 17,
         'user_functions': {},
-        'human': True,
         'contract': True,
         'dereference': set(),
-        'error_on_reserved': False,
-        'reserved_word_suffix': '_',
-        'inline': False,
-    }
+    })
 
     def __init__(self, settings={}):
         CodePrinter.__init__(self, settings)
@@ -299,7 +295,7 @@ class RustCodePrinter(CodePrinter):
             cond_func = self.known_functions[expr.func.__name__]
             func = None
             style = 1
-            if isinstance(cond_func, string_types):
+            if isinstance(cond_func, str):
                 func = cond_func
             else:
                 for cond, func, style in cond_func:
@@ -331,6 +327,11 @@ class RustCodePrinter(CodePrinter):
         elif hasattr(expr, '_imp_') and isinstance(expr._imp_, Lambda):
             # inlined function
             return self._print(expr._imp_(*expr.args))
+        elif expr.func.__name__ in self._rewriteable_functions:
+            # Simple rewrite to supported function possible
+            target_f, required_fs = self._rewriteable_functions[expr.func.__name__]
+            if self._can_print(target_f) and all(self._can_print(f) for f in required_fs):
+                return self._print(expr.rewrite(target_f))
         else:
             return self._print_not_supported(expr)
 
@@ -341,14 +342,14 @@ class RustCodePrinter(CodePrinter):
         return self._print_Function(expr)
 
     def _print_Float(self, expr, _type=False):
-        ret = super(RustCodePrinter, self)._print_Float(expr)
+        ret = super()._print_Float(expr)
         if _type:
             return ret + '_f64'
         else:
             return ret
 
     def _print_Integer(self, expr, _type=False):
-        ret = super(RustCodePrinter, self)._print_Integer(expr)
+        ret = super()._print_Integer(expr)
         if _type:
             return ret + '_i32'
         else:
@@ -362,7 +363,7 @@ class RustCodePrinter(CodePrinter):
         lhs_code = self._print(expr.lhs)
         rhs_code = self._print(expr.rhs)
         op = expr.rel_op
-        return "{0} {1} {2}".format(lhs_code, op, rhs_code)
+        return "{} {} {}".format(lhs_code, op, rhs_code)
 
     def _print_Indexed(self, expr):
         # calculate index for 1d array
@@ -433,12 +434,7 @@ class RustCodePrinter(CodePrinter):
 
     def _print_ITE(self, expr):
         from sympy.functions import Piecewise
-        _piecewise = Piecewise((expr.args[1], expr.args[0]), (expr.args[2], True))
-        return self._print(_piecewise)
-
-    def _print_Matrix(self, expr):
-        return "%s[%s]" % (expr.parent,
-                           expr.j + expr.i*expr.parent.shape[1])
+        return self._print(expr.rewrite(Piecewise, deep=False))
 
     def _print_MatrixBase(self, A):
         if A.cols == 1:
@@ -446,23 +442,17 @@ class RustCodePrinter(CodePrinter):
         else:
             raise ValueError("Full Matrix Support in Rust need Crates (https://crates.io/keywords/matrix).")
 
+    def _print_SparseRepMatrix(self, mat):
+        # do not allow sparse matrices to be made dense
+        return self._print_not_supported(mat)
+
     def _print_MatrixElement(self, expr):
         return "%s[%s]" % (expr.parent,
                            expr.j + expr.i*expr.parent.shape[1])
 
-    # FIXME: Str/CodePrinter could define each of these to call the _print
-    # method from higher up the class hierarchy (see _print_NumberSymbol).
-    # Then subclasses like us would not need to repeat all this.
-    _print_Matrix = \
-        _print_DenseMatrix = \
-        _print_MutableDenseMatrix = \
-        _print_ImmutableMatrix = \
-        _print_ImmutableDenseMatrix = \
-        _print_MatrixBase
-
     def _print_Symbol(self, expr):
 
-        name = super(RustCodePrinter, self)._print_Symbol(expr)
+        name = super()._print_Symbol(expr)
 
         if expr in self._dereference:
             return '(*%s)' % name
@@ -486,7 +476,7 @@ class RustCodePrinter(CodePrinter):
     def indent_code(self, code):
         """Accepts a string of code or a list of code lines"""
 
-        if isinstance(code, string_types):
+        if isinstance(code, str):
             code_lines = self.indent_code(code.splitlines(True))
             return ''.join(code_lines)
 
@@ -503,7 +493,7 @@ class RustCodePrinter(CodePrinter):
         pretty = []
         level = 0
         for n, line in enumerate(code):
-            if line == '' or line == '\n':
+            if line in ('', '\n'):
                 pretty.append(line)
                 continue
             level -= decrease[n]
@@ -519,7 +509,7 @@ def rust_code(expr, assign_to=None, **settings):
     ==========
 
     expr : Expr
-        A sympy expression to be converted.
+        A SymPy expression to be converted.
     assign_to : optional
         When given, the argument is used as the name of the variable to which
         the expression is assigned. Can be a string, ``Symbol``,
