@@ -1,24 +1,25 @@
 """
-A Printer for generating readable representation of most sympy classes.
+A Printer for generating readable representation of most SymPy classes.
 """
 
-from typing import Any, Dict
+from __future__ import annotations
+from typing import Any
 
 from sympy.core import S, Rational, Pow, Basic, Mul, Number
 from sympy.core.mul import _keep_coeff
-from sympy.core.function import _coeff_isneg
-from sympy.sets.sets import FiniteSet
+from sympy.core.numbers import Integer
+from sympy.core.relational import Relational
+from sympy.core.sorting import default_sort_key
+from sympy.utilities.iterables import sift
+from .precedence import precedence, PRECEDENCE
 from .printer import Printer, print_function
-from sympy.printing.precedence import precedence, PRECEDENCE
 
 from mpmath.libmp import prec_to_dps, to_str as mlib_to_str
 
-from sympy.utilities import default_sort_key, sift
-
 
 class StrPrinter(Printer):
     printmethod = "_sympystr"
-    _default_settings = {
+    _default_settings: dict[str, Any] = {
         "order": None,
         "full_prec": "auto",
         "sympy_integers": False,
@@ -26,9 +27,9 @@ class StrPrinter(Printer):
         "perm_cyclic": True,
         "min": None,
         "max": None,
-    }  # type: Dict[str, Any]
+    }
 
-    _relationals = dict()  # type: Dict[str, str]
+    _relationals: dict[str, str] = {}
 
     def parenthesize(self, item, level, strict=False):
         if (precedence(item) < level) or ((not strict) and precedence(item) <= level):
@@ -50,16 +51,16 @@ class StrPrinter(Printer):
     def _print_Add(self, expr, order=None):
         terms = self._as_ordered_terms(expr, order=order)
 
-        PREC = precedence(expr)
+        prec = precedence(expr)
         l = []
         for term in terms:
             t = self._print(term)
-            if t.startswith('-'):
+            if t.startswith('-') and not term.is_Add:
                 sign = "-"
                 t = t[1:]
             else:
                 sign = "+"
-            if precedence(term) < PREC:
+            if precedence(term) < prec or term.is_Add:
                 l.extend([sign, "(%s)" % t])
             else:
                 l.extend([sign, t])
@@ -78,7 +79,12 @@ class StrPrinter(Printer):
         return '~%s' %(self.parenthesize(expr.args[0],PRECEDENCE["Not"]))
 
     def _print_And(self, expr):
-        return self.stringify(expr.args, " & ", PRECEDENCE["BitwiseAnd"])
+        args = list(expr.args)
+        for j, i in enumerate(args):
+            if isinstance(i, Relational) and (
+                    i.canonical.rhs is S.NegativeInfinity):
+                args.insert(0, args.pop(j))
+        return self.stringify(args, " & ", PRECEDENCE["BitwiseAnd"])
 
     def _print_Or(self, expr):
         return self.stringify(expr.args, " | ", PRECEDENCE["BitwiseOr"])
@@ -115,7 +121,7 @@ class StrPrinter(Printer):
     def _print_Derivative(self, expr):
         dexpr = expr.expr
         dvars = [i[0] if i[1] == 1 else i for i in expr.variable_count]
-        return 'Derivative(%s)' % ", ".join(map(lambda arg: self._print(arg), [dexpr] + dvars))
+        return 'Derivative(%s)' % ", ".join((self._print(arg) for arg in [dexpr] + dvars))
 
     def _print_dict(self, d):
         keys = sorted(d.keys(), key=default_sort_key)
@@ -219,15 +225,15 @@ class StrPrinter(Printer):
 
     def _print_Limit(self, expr):
         e, z, z0, dir = expr.args
-        if str(dir) == "+":
-            return "Limit(%s, %s, %s)" % tuple(map(self._print, (e, z, z0)))
-        else:
-            return "Limit(%s, %s, %s, dir='%s')" % tuple(map(self._print,
-                                                            (e, z, z0, dir)))
+        return "Limit(%s, %s, %s, dir='%s')" % tuple(map(self._print, (e, z, z0, dir)))
+
 
     def _print_list(self, expr):
         return "[%s]" % self.stringify(expr, ", ")
 
+    def _print_List(self, expr):
+        return self._print_list(expr)
+
     def _print_MatrixBase(self, expr):
         return expr._format_str(self)
 
@@ -244,7 +250,7 @@ class StrPrinter(Printer):
                 x[0] = ''
             if x[1] == dim:
                 x[1] = ''
-            return ':'.join(map(lambda arg: self._print(arg), x))
+            return ':'.join((self._print(arg) for arg in x))
         return (self.parenthesize(expr.parent, PRECEDENCE["Atom"], strict=True) + '[' +
                 strslice(expr.rowslice, expr.parent.rows) + ', ' +
                 strslice(expr.colslice, expr.parent.cols) + ']')
@@ -277,17 +283,19 @@ class StrPrinter(Printer):
                     e = Mul._from_args(dargs)
                 d[i] = Pow(di.base, e, evaluate=False) if e - 1 else di.base
 
+            pre = []
             # don't parenthesize first factor if negative
-            if _coeff_isneg(n[0]):
-                pre = [str(n.pop(0))]
-            else:
-                pre = []
+            if n and not n[0].is_Add and n[0].could_extract_minus_sign():
+                pre = [self._print(n.pop(0))]
+
             nfactors = pre + [self.parenthesize(a, prec, strict=False)
                 for a in n]
+            if not nfactors:
+                nfactors = ['1']
 
             # don't parenthesize first of denominator unless singleton
-            if len(d) > 1 and _coeff_isneg(d[0]):
-                pre = [str(d.pop(0))]
+            if len(d) > 1 and d[0].could_extract_minus_sign():
+                pre = [self._print(d.pop(0))]
             else:
                 pre = []
             dfactors = pre + [self.parenthesize(a, prec, strict=False)
@@ -414,16 +422,19 @@ class StrPrinter(Printer):
 
     def _print_Permutation(self, expr):
         from sympy.combinatorics.permutations import Permutation, Cycle
-        from sympy.utilities.exceptions import SymPyDeprecationWarning
+        from sympy.utilities.exceptions import sympy_deprecation_warning
 
         perm_cyclic = Permutation.print_cyclic
         if perm_cyclic is not None:
-            SymPyDeprecationWarning(
-                feature="Permutation.print_cyclic = {}".format(perm_cyclic),
-                useinstead="init_printing(perm_cyclic={})"
-                .format(perm_cyclic),
-                issue=15201,
-                deprecated_since_version="1.6").warn()
+            sympy_deprecation_warning(
+                f"""
+                Setting Permutation.print_cyclic is deprecated. Instead use
+                init_printing(perm_cyclic={perm_cyclic}).
+                """,
+                deprecated_since_version="1.6",
+                active_deprecations_target="deprecated-permutation-print_cyclic",
+                stacklevel=7,
+            )
         else:
             perm_cyclic = self._settings.get("perm_cyclic", True)
 
@@ -481,7 +492,8 @@ class StrPrinter(Printer):
         return self._print(expr.name)
 
     def _print_ArrayElement(self, expr):
-        return "%s[%s]" % (expr.name, ", ".join([self._print(i) for i in expr.indices]))
+        return "%s[%s]" % (
+            self.parenthesize(expr.name, PRECEDENCE["Func"], True), ", ".join([self._print(i) for i in expr.indices]))
 
     def _print_PermutationGroup(self, expr):
         p = ['    %s' % self._print(a) for a in expr.args]
@@ -492,12 +504,12 @@ class StrPrinter(Printer):
 
     def _print_PolyRing(self, ring):
         return "Polynomial ring in %s over %s with %s order" % \
-            (", ".join(map(lambda rs: self._print(rs), ring.symbols)),
+            (", ".join((self._print(rs) for rs in ring.symbols)),
             self._print(ring.domain), self._print(ring.order))
 
     def _print_FracField(self, field):
         return "Rational function field in %s over %s with %s order" % \
-            (", ".join(map(lambda fs: self._print(fs), field.symbols)),
+            (", ".join((self._print(fs) for fs in field.symbols)),
             self._print(field.domain), self._print(field.order))
 
     def _print_FreeGroupElement(self, elm):
@@ -560,7 +572,7 @@ class StrPrinter(Printer):
             else:
                 terms.extend(['+', s_term])
 
-        if terms[0] in ['-', '+']:
+        if terms[0] in ('-', '+'):
             modifier = terms.pop(0)
 
             if modifier == '-':
@@ -608,8 +620,7 @@ class StrPrinter(Printer):
         Examples
         ========
 
-        >>> from sympy.functions import sqrt
-        >>> from sympy.printing.str import StrPrinter
+        >>> from sympy import sqrt, StrPrinter
         >>> from sympy.abc import x
 
         How ``rational`` keyword works with ``sqrt``:
@@ -640,7 +651,7 @@ class StrPrinter(Printer):
             if -expr.exp is S.Half and not rational:
                 # Note: Don't test "expr.exp == -S.Half" here, because that will
                 # match -0.5, which we don't want.
-                return "%s/sqrt(%s)" % tuple(map(lambda arg: self._print(arg), (S.One, expr.base)))
+                return "%s/sqrt(%s)" % tuple((self._print(arg) for arg in (S.One, expr.base)))
             if expr.exp is -S.One:
                 # Similarly to the S.Half case, don't test with "==" here.
                 return '%s/%s' % (self._print(S.One),
@@ -803,6 +814,7 @@ class StrPrinter(Printer):
         return '{%s}' % args
 
     def _print_FiniteSet(self, s):
+        from sympy.sets.sets import FiniteSet
         items = sorted(s, key=default_sort_key)
 
         args = ', '.join(self._print(item) for item in items)
@@ -896,26 +908,22 @@ class StrPrinter(Printer):
     def _print_Zero(self, expr):
         if self._settings.get("sympy_integers", False):
             return "S(0)"
-        return "0"
+        return self._print_Integer(Integer(0))
 
     def _print_DMP(self, p):
-        from sympy.core.sympify import SympifyError
-        try:
-            if p.ring is not None:
-                # TODO incorporate order
-                return self._print(p.ring.to_sympy(p))
-        except SympifyError:
-            pass
-
         cls = p.__class__.__name__
-        rep = self._print(p.rep)
+        rep = self._print(p.to_list())
         dom = self._print(p.dom)
-        ring = self._print(p.ring)
 
-        return "%s(%s, %s, %s)" % (cls, rep, dom, ring)
+        return "%s(%s, %s)" % (cls, rep, dom)
 
     def _print_DMF(self, expr):
-        return self._print_DMP(expr)
+        cls = expr.__class__.__name__
+        num = self._print(expr.num)
+        den = self._print(expr.den)
+        dom = self._print(expr.dom)
+
+        return "%s(%s, %s, %s)" % (cls, num, den, dom)
 
     def _print_Object(self, obj):
         return 'Object("%s")' % obj.name
