@@ -1,7 +1,7 @@
-from sympy.printing.codeprinter import CodePrinter
+from sympy.printing.codeprinter import CodePrinter, PrintMethodNotImplementedError
 from sympy.core import symbols
 from sympy.core.symbol import Dummy
-from sympy.utilities.pytest import raises
+from sympy.testing.pytest import raises
 
 
 def setup_test_printer(**kwargs):
@@ -35,3 +35,21 @@ def test_print_Symbol():
     p = setup_test_printer(reserved_word_suffix='_He_Man')
     p.reserved_words.update(['if'])
     assert p._print(y) == 'if_He_Man'
+
+def test_issue_15791():
+    class CrashingCodePrinter(CodePrinter):
+        def emptyPrinter(self, obj):
+            raise NotImplementedError
+
+    from sympy.matrices import (
+        MutableSparseMatrix,
+        ImmutableSparseMatrix,
+    )
+
+    c = CrashingCodePrinter()
+
+    # these should not silently succeed
+    with raises(PrintMethodNotImplementedError):
+        c.doprint(ImmutableSparseMatrix(2, 2, {}))
+    with raises(PrintMethodNotImplementedError):
+        c.doprint(MutableSparseMatrix(2, 2, {}))
