@@ -1,14 +1,14 @@
 from sympy.core.add import Add
-from sympy.core.compatibility import ordered
-from sympy.core.function import expand_log
+from sympy.core.exprtools import factor_terms
+from sympy.core.function import expand_log, _mexpand
 from sympy.core.power import Pow
 from sympy.core.singleton import S
+from sympy.core.sorting import ordered
 from sympy.core.symbol import Dummy
 from sympy.functions.elementary.exponential import (LambertW, exp, log)
 from sympy.functions.elementary.miscellaneous import root
 from sympy.polys.polyroots import roots
 from sympy.polys.polytools import Poly, factor
-from sympy.core.function import _mexpand
 from sympy.simplify.simplify import separatevars
 from sympy.simplify.radsimp import collect
 from sympy.simplify.simplify import powsimp
@@ -31,6 +31,9 @@ def _filtered_gens(poly, symbol):
     {x, exp(x)}
 
     """
+    # TODO it would be good to pick the smallest divisible power
+    # instead of the base for something like x**4 + x**2 -->
+    # return x**2 not x
     gens = {g for g in poly.gens if symbol in g.free_symbols}
     for g in list(gens):
         ag = 1/g
@@ -55,7 +58,7 @@ def _mostfunc(lhs, func, X=None):
     ========
 
     >>> from sympy.solvers.bivariate import _mostfunc
-    >>> from sympy.functions.elementary.exponential import exp
+    >>> from sympy import exp
     >>> from sympy.abc import x, y
     >>> _mostfunc(exp(x) + exp(exp(x) + 2), exp)
     exp(exp(x) + 2)
@@ -86,10 +89,9 @@ def _linab(arg, symbol):
     Examples
     ========
 
-    >>> from sympy.functions.elementary.exponential import exp
     >>> from sympy.solvers.bivariate import _linab
     >>> from sympy.abc import x, y
-    >>> from sympy import S
+    >>> from sympy import exp, S
     >>> _linab(S(2), x)
     (2, 0, 1)
     >>> _linab(2*x, x)
@@ -99,7 +101,6 @@ def _linab(arg, symbol):
     >>> _linab(3 + 2*exp(x), x)
     (2, 3, exp(x))
     """
-    from sympy.core.exprtools import factor_terms
     arg = factor_terms(arg.expand())
     ind, dep = arg.as_independent(symbol)
     if arg.is_Mul and dep.is_Add:
@@ -136,7 +137,7 @@ def _lambert(eq, x):
             return []  # violated assumptions
         other = -(-other).args[0]
         eq += other
-    if not x in other.free_symbols:
+    if x not in other.free_symbols:
         return [] # violated assumptions
     d, f, X2 = _linab(other, x)
     logterm = collect(eq - other, mainlog)
@@ -185,8 +186,7 @@ def _lambert(eq, x):
                 continue
             rhs = -c/b + (a/d)*w
 
-            for xu in xusolns:
-                sol.append(xu.subs(u, rhs))
+            sol.extend(xu.subs(u, rhs) for xu in xusolns)
     return sol
 
 
@@ -260,7 +260,7 @@ def _solve_lambert(f, symbol, gens):
         solutions for  ``2*log(-x) + log(g(x))`` since those must also
         be a solution of ``eq`` which has the same value when the ``x``
         in ``x**2`` is negated. If `g(x)` does not have even powers of
-        symbol then we don't want to replace the ``x`` there with
+        symbol then we do not want to replace the ``x`` there with
         ``-x``. So the role of the ``t`` in the expression received by
         this function is to mark where ``+/-x`` should be inserted
         before obtaining the Lambert solutions.
@@ -273,7 +273,7 @@ def _solve_lambert(f, symbol, gens):
             sols.extend(_solve_lambert(plhs, symbol, gens))
         # uniq is needed for a case like
         # 2*log(t) - log(-z**2) + log(z + log(x) + log(z))
-        # where subtituting t with +/-x gives all the same solution;
+        # where substituting t with +/-x gives all the same solution;
         # uniq, rather than list(set()), is used to maintain canonical
         # order
         return list(uniq(sols))
@@ -437,7 +437,7 @@ def bivariate_type(f, x, y, *, first=True):
     Examples
     ========
 
-    >>> from sympy.solvers.solvers import solve
+    >>> from sympy import solve
     >>> from sympy.solvers.bivariate import bivariate_type
     >>> from sympy.abc import x, y
     >>> eq = (x**2 - 3).subs(x, x + y)
