@@ -4,7 +4,7 @@ from collections import defaultdict, Counter
 from functools import reduce
 import itertools
 from itertools import accumulate
-from typing import Optional, List, Dict as tDict, Tuple as tTuple
+from typing import Optional, List, Tuple as tTuple
 
 import typing
 
@@ -19,7 +19,7 @@ from sympy.core.mul import Mul
 from sympy.core.singleton import S
 from sympy.core.sorting import default_sort_key
 from sympy.core.symbol import (Dummy, Symbol)
-from sympy.matrices.common import MatrixCommon
+from sympy.matrices.matrixbase import MatrixBase
 from sympy.matrices.expressions.diagonal import diagonalize_vector
 from sympy.matrices.expressions.matexpr import MatrixExpr
 from sympy.matrices.expressions.special import ZeroMatrix
@@ -395,7 +395,7 @@ class PermuteDims(_CodegenArrayAbstract):
 
     This is evident when transforming back to matrix form:
 
-    >>> from sympy.tensor.array.expressions.conv_array_to_matrix import convert_array_to_matrix
+    >>> from sympy.tensor.array.expressions.from_array_to_matrix import convert_array_to_matrix
     >>> convert_array_to_matrix(cg)
     M.T
 
@@ -553,7 +553,7 @@ class PermuteDims(_CodegenArrayAbstract):
             permutation_array_blocks_up.append(current)
 
         # Get the map of axis repositioning for every argument of tensor-product:
-        index_blocks = [[j for j in range(cumul[i], cumul[i+1])] for i, e in enumerate(expr.subranks)]
+        index_blocks = [list(range(cumul[i], cumul[i+1])) for i, e in enumerate(expr.subranks)]
         index_blocks_up = expr._push_indices_up(expr.contraction_indices, index_blocks)
         inverse_permutation = permutation**(-1)
         index_blocks_up_permuted = [[inverse_permutation(j) for j in i if j is not None] for i in index_blocks_up]
@@ -583,7 +583,7 @@ class PermuteDims(_CodegenArrayAbstract):
         arg_candidate_index = index2arg[permuted_indices[0]]
         current_indices = []
         new_permutation = []
-        inserted_arg_cand_indices = set([])
+        inserted_arg_cand_indices = set()
         for i, idx in enumerate(permuted_indices):
             if index2arg[idx] != arg_candidate_index:
                 new_permutation.extend(current_indices)
@@ -607,7 +607,7 @@ class PermuteDims(_CodegenArrayAbstract):
         maps = {}
         cumulative_subranks = [0] + list(accumulate(subranks))
         for i in range(len(subranks)):
-            s = set([index2arg[new_permutation[j]] for j in range(cumulative_subranks[i], cumulative_subranks[i+1])])
+            s = {index2arg[new_permutation[j]] for j in range(cumulative_subranks[i], cumulative_subranks[i+1])}
             if len(s) != 1:
                 continue
             elem = next(iter(s))
@@ -779,7 +779,7 @@ class ArrayDiagonal(_CodegenArrayAbstract):
             rank2 = len(diagonal_indices)
             rank3 = rank1 - rank2
             inv_permutation = []
-            counter1: int = 0
+            counter1 = 0
             indices_down = ArrayDiagonal._push_indices_down(diagonal_indices_short, list(range(rank1)), get_rank(expr))
             for i in indices_down:
                 if i in trivial_pos:
@@ -1078,8 +1078,8 @@ class ArrayContraction(_CodegenArrayAbstract):
         cumranks = list(accumulate([0] + subranks))
         contraction_indices_remaining = []
         contraction_indices_args = [[] for i in expr.args]
-        backshift = set([])
-        for i, contraction_group in enumerate(contraction_indices):
+        backshift = set()
+        for contraction_group in contraction_indices:
             for j in range(len(expr.args)):
                 if not isinstance(expr.args[j], ArrayAdd):
                     continue
@@ -1154,8 +1154,8 @@ class ArrayContraction(_CodegenArrayAbstract):
             # Also consider the case of diagonal matrices being contracted:
             current_dimension = self.expr.shape[links[0]]
 
-            not_vectors: tTuple[_ArgE, int] = []
-            vectors: tTuple[_ArgE, int] = []
+            not_vectors = []
+            vectors = []
             for arg_ind, rel_ind in positions:
                 arg = editor.args_with_ind[arg_ind]
                 mat = arg.element
@@ -1426,7 +1426,7 @@ class ArrayContraction(_CodegenArrayAbstract):
         Examples
         ========
 
-        >>> from sympy.tensor.array.expressions.conv_matrix_to_array import convert_matrix_to_array
+        >>> from sympy.tensor.array.expressions.from_matrix_to_array import convert_matrix_to_array
         >>> from sympy import MatrixSymbol
         >>> from sympy.abc import N
         >>> A = MatrixSymbol("A", N, N)
@@ -1468,7 +1468,7 @@ class ArrayContraction(_CodegenArrayAbstract):
 
         >>> from sympy import MatrixSymbol
         >>> from sympy.abc import N
-        >>> from sympy.tensor.array.expressions.conv_matrix_to_array import convert_matrix_to_array
+        >>> from sympy.tensor.array.expressions.from_matrix_to_array import convert_matrix_to_array
         >>> A = MatrixSymbol("A", N, N)
         >>> B = MatrixSymbol("B", N, N)
         >>> C = MatrixSymbol("C", N, N)
@@ -1554,7 +1554,7 @@ class Reshape(_CodegenArrayAbstract):
             expr = self.expr.doit(*args, **kwargs)
         else:
             expr = self.expr
-        if isinstance(expr, (MatrixCommon, NDimArray)):
+        if isinstance(expr, (MatrixBase, NDimArray)):
             return expr.reshape(*self.shape)
         return Reshape(expr, self.shape)
 
@@ -1562,7 +1562,7 @@ class Reshape(_CodegenArrayAbstract):
         ee = self.expr
         if hasattr(ee, "as_explicit"):
             ee = ee.as_explicit()
-        if isinstance(ee, MatrixCommon):
+        if isinstance(ee, MatrixBase):
             from sympy import Array
             ee = Array(ee)
         elif isinstance(ee, MatrixExpr):
@@ -1702,12 +1702,12 @@ class _EditArrayContraction:
         return self.number_of_contraction_indices - 1
 
     def refresh_indices(self):
-        updates: tDict[int, int] = {}
+        updates = {}
         for arg_with_ind in self.args_with_ind:
             updates.update({i: -1 for i in arg_with_ind.indices if i is not None})
         for i, e in enumerate(sorted(updates)):
             updates[e] = i
-        self.number_of_contraction_indices: int = len(updates)
+        self.number_of_contraction_indices = len(updates)
         for arg_with_ind in self.args_with_ind:
             arg_with_ind.indices = [updates.get(i, None) for i in arg_with_ind.indices]
 
@@ -1722,7 +1722,7 @@ class _EditArrayContraction:
         if len(self.args_with_ind) == 0:
             self.args_with_ind.append(_ArgE(scalar))
         else:
-            from sympy.tensor.array.expressions.conv_array_to_matrix import _a2m_tensor_product
+            from sympy.tensor.array.expressions.from_array_to_matrix import _a2m_tensor_product
             self.args_with_ind[0].element = _a2m_tensor_product(scalar, self.args_with_ind[0].element)
 
     def to_array_contraction(self):
@@ -1742,7 +1742,7 @@ class _EditArrayContraction:
         inv_perm1 = []
         inv_perm2 = []
         # Keep track of which diagonal indices have already been processed:
-        done = set([])
+        done = set()
 
         # Counter for the diagonal indices:
         counter4 = 0
@@ -1795,7 +1795,7 @@ class _EditArrayContraction:
     def get_contraction_indices(self) -> List[List[int]]:
         contraction_indices: List[List[int]] = [[] for i in range(self.number_of_contraction_indices)]
         current_position: int = 0
-        for i, arg_with_ind in enumerate(self.args_with_ind):
+        for arg_with_ind in self.args_with_ind:
             for j in arg_with_ind.indices:
                 if j is not None:
                     contraction_indices[j].append(current_position)
@@ -1839,7 +1839,7 @@ class _EditArrayContraction:
 
     @property
     def number_of_diagonal_indices(self):
-        data = set([])
+        data = set()
         for arg in self.args_with_ind:
             data.update({i for i in arg.indices if i is not None and i < 0})
         return len(data)
@@ -1847,8 +1847,8 @@ class _EditArrayContraction:
     def track_permutation_start(self):
         permutation = []
         perm_diag = []
-        counter: int = 0
-        counter2: int = -1
+        counter = 0
+        counter2 = -1
         for arg_with_ind in self.args_with_ind:
             perm = []
             for i in arg_with_ind.indices:
@@ -1860,7 +1860,7 @@ class _EditArrayContraction:
                 perm.append(counter)
                 counter += 1
             permutation.append(perm)
-        max_ind = max([max(i) if i else -1 for i in permutation]) if permutation else -1
+        max_ind = max(max(i) if i else -1 for i in permutation) if permutation else -1
         perm_diag = [max_ind - i for i in perm_diag]
         self._track_permutation = permutation + [perm_diag]
 
