@@ -1,4 +1,4 @@
-from sympy.testing.pytest import warns_deprecated_sympy, XFAIL
+from sympy.testing.pytest import warns_deprecated_sympy
 
 # See https://github.com/sympy/sympy/pull/18095
 
@@ -11,9 +11,3 @@ def test_deprecated_utilities():
         import sympy.utilities.randtest  # noqa:F401
     with warns_deprecated_sympy():
         import sympy.utilities.tmpfiles  # noqa:F401
-
-# This fails because benchmarking isn't importable...
-@XFAIL
-def test_deprecated_benchmarking():
-    with warns_deprecated_sympy():
-        import sympy.utilities.benchmarking  # noqa:F401
